@@ -1,3 +1,5 @@
+(** Constants describing configuration options of liquidsoap. *)
+
 (** String describing the version. *)
 val version : string
 
@@ -17,6 +19,10 @@ val recode_tag : ?encoding:string -> string -> string
 (** Maximal id for a request. *)
 val requests_max_id : int
 
+(** Magic mime detection *)
+val file_mime : string -> string option
+val data_mime : ?len:int -> string -> string option
+
 val requests_table_size : int
 
 (** Program used for text-to-speech. *)
