@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ type decoder = {
 (** Plugins are given filenames and return a decoder, if possible. *)
 let formats : (string -> decoder option) Plug.plug =
   Plug.create
-    ~doc:"Method to read audio files." ~insensitive:true "file formats"
+    ~doc:"Method to read audio files." ~insensitive:true "audio file formats"
 
 (** How much data (in standard samples) should we decode in advance ?
   * This value affects the accuracy of the remaining time estimations. *)
@@ -69,7 +69,7 @@ exception Exit of (string * (string -> decoder option))
   * Being based on file extension is weak, and troublesome when accessing a
   * remote file -- that would force us to create a local temporary file with the
   * same extension. *)
-let search_valid filename : (unit -> decoder) option =
+let search_valid filename : (string*(unit -> decoder)) option =
   try
     formats#iter
       (fun format decoder ->
@@ -83,7 +83,7 @@ let search_valid filename : (unit -> decoder) option =
     None
   with
     | Exit (format,d) ->
-        Some (fun () ->
+        Some (format,fun () ->
                 match d filename with
                   | None ->
                       log#f 2 "Decoder %s betrayed us on %S!" format filename ;
