@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,8 +28,32 @@ module Generator = Float_pcm.Generator
 
 let log = Log.make ["format";"mp3"]
 
-let decoder file =
+let conf_mad =
+  Conf.void ~p:(Configure.conf#plug "mad")
+    "Mad (mp3) decoding options."
+let conf_mime =
+  Conf.bool ~p:(conf_mad#plug "check_mime") ~d:true
+      "Enable magic mime test, if compiled."
+let conf_mime_types =
+  Conf.list ~p:(conf_mad#plug "mime_types")
+    "Mime-types used for guessing formats"
+    ~d:["audio/mpeg";"application/octet-stream";"video/x-unknown"]
 
+let decoder file =
+  begin
+    if conf_mime#get then
+      match Configure.file_mime file with
+        (* libmagic does not always detect mp3 as audio/mpeg
+         * (Repeat after me: mp3 formats sucks, mp3 format.... )
+         * So it might return other types.
+         * We hope other formats like jpeg, gif, etc.. 
+         * will most likely be correctly detected *)
+        | Some s when List.mem s conf_mime_types#get -> ()
+        | None -> ()
+        | Some s ->
+          log#f 2 "Mime type for %s is not valid: %s" file s;
+          assert false
+  end ;
   let fd =
     log#f 4 "open %S" file ;
     Mad.openfile file
@@ -86,5 +110,12 @@ let decoder file =
   in
     { Decoder.fill = fill ; Decoder.close = close }
 
+let duration file =
+  let ans = Mad.duration file in
+  match ans with
+    | 0. -> raise Not_found
+    | _ -> ans
+
 let () = Decoder.formats#register "MP3"
-           (fun name -> try Some (decoder name) with _ -> None)
+           (fun name -> try Some (decoder name) with _ -> None);
+         Request.dresolvers#register "MP3" duration
