@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -250,7 +250,7 @@ let print t =
             let constraints =
               List.map
                 (fun (ids,c) ->
-                   Printf.sprintf "%s is %s" (String.concat "/" ids) c)
+                   Printf.sprintf "%s is %s" (String.concat ", " ids) c)
                 constraints
             in
               repr ^ " where " ^
@@ -272,7 +272,8 @@ let fresh_evar =
 exception Occur_check of t*t
 exception Unsatisfied_constraint of constr*t
 
-(** Check that [a] (a dereferenced type variable) does not occur in [b]. *)
+(** Check that [a] (a dereferenced type variable) does not occur in [b],
+  * and prepare the instantiation [a<-b] by adjusting the levels. *)
 let rec occur_check a b =
   let b = deref b in
     if a == b then raise (Occur_check (a,b)) ;
@@ -284,7 +285,9 @@ let rec occur_check a b =
             (fun (o,l,t) -> occur_check a t)
             p ;
           occur_check a t
-      | EVar _ -> b.level <- min b.level a.level
+      | EVar _ ->
+          assert (a.level <> -1 && b.level <> -1) ;
+          b.level <- min b.level a.level
       | UVar _ -> assert false
       | Link _ | Ground _ -> ()
 
@@ -303,7 +306,7 @@ let rec bind a0 b =
                        begin match b.descr with
                          | Ground g' -> if g<>g' then raise error
                          | Arrow([],t) ->
-                             begin match t.descr with
+                             begin match (deref t).descr with
                                | Ground g' -> if g<>g' then raise error
                                | EVar (j,c) ->
                                    (* This is almost wrong as it flips <: into
@@ -338,7 +341,7 @@ let rec bind a0 b =
                            if not (List.mem g [Bool;Int;Float;String]) then
                              raise (Unsatisfied_constraint (Dtools,b))
                        | List b' ->
-                           begin match b'.descr with
+                           begin match (deref b').descr with
                              | Ground g ->
                                  if g <> String then
                                    raise (Unsatisfied_constraint (Dtools,b'))
@@ -469,9 +472,13 @@ let rec (<:) a b =
      * subtyping constraint instead of unifying. Nevermind...
      * It's a pain for arrow types, and forgetting about it doesn't hurt. *)
     | EVar _, _ ->
-        begin try bind a b with _ -> raise (Error [Item (a,b)]) end
+        begin try bind a b with
+          | Occur_check _ | Unsatisfied_constraint _ ->
+              raise (Error [Item (a,b)]) end
     | _, EVar _ ->
-        begin try bind b a with _ -> raise (Error [Item (a,b)]) end
+        begin try bind b a with
+          | Occur_check _ | Unsatisfied_constraint _ ->
+              raise (Error [Item (a,b)]) end
     | UVar _,_ | _,UVar _ -> assert false (* no type schemes here *)
     | Link _,_ | _,Link _ -> assert false (* thanks to deref *)
     | Ground x,Ground y ->
