@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,10 +25,10 @@ open Dtools
 open Printf
 
 let usage =
-  "Usage : liquidsoap [OPTION, SCRIPT or EXPR]...\n" ^
-  " - SCRIPT for evaluating a liquidsoap script file;\n" ^
-  " - EXPR for evaluating a scripting expression;\n" ^
-  " - OPTION is one of the options listed below:\n"
+  "Usage : liquidsoap [OPTION, SCRIPT or EXPR]...\n\
+  \ - SCRIPT for evaluating a liquidsoap script file;\n\
+  \ - EXPR for evaluating a scripting expression;\n\
+  \ - OPTION is one of the options listed below:\n"
 
 let () =
   Configure.conf#plug "init" Init.conf ;
@@ -37,6 +37,9 @@ let () =
 (* Should we not_run the outputs? *)
 let dont_run = ref false
 
+(* Do not run, don't even check the scripts. *)
+let parse_only = ref false
+
 (* Should we load the pervasives? *)
 let pervasives = ref true
 
@@ -50,7 +53,7 @@ let check_pervasives =
   let loaded = ref false in
     fun () ->
       if !pervasives && not !loaded then begin
-        Lang.load_libs () ;
+        Lang.load_libs ~parse_only:!parse_only () ;
         loaded := true
       end
 
@@ -84,18 +87,18 @@ let eval src =
     | `StdIn ->
         Log.conf_stdout#set_d (Some true) ;
         Log.conf_file#set_d (Some false) ;
-        Lang.from_in_channel stdin
+        Lang.from_in_channel ~parse_only:!parse_only stdin
     | `Expr_or_File expr when (not (Sys.file_exists expr)) ->
         Log.conf_stdout#set_d (Some true) ;
         Log.conf_file#set_d (Some false) ;
-        Lang.from_string expr
+        Lang.from_string ~parse_only:!parse_only expr
     | `Expr_or_File f ->
         let basename = Filename.basename f in
         let basename =
           try Filename.chop_extension basename with _ -> basename
         in
           Configure.var_script := basename ;
-          Lang.from_file f
+          Lang.from_file ~parse_only:!parse_only f
 
 let process_request s =
   secondary_task := true ;
@@ -113,6 +116,13 @@ let process_request s =
           let metadata = Request.get_all_metadata req in
           let metadata = Request.string_of_metadata metadata in
             Printf.printf "Request resolved.\n%s\n" metadata ;
+            Printf.printf "Computing duration: %!" ;
+            begin try
+              Printf.printf "%.2f sec.\n"
+                (Request.duration (Utils.get_some (Request.get_filename req)))
+            with
+              Not_found -> Printf.printf "failed.\n"
+            end ;
             Request.destroy req
 
 module LiqConf =
@@ -279,8 +289,16 @@ let options =
       "Print the description of a plugin.";
 
       ["-c";"--check"],
-      Arg.Set dont_run,
-      "Evaluate scripts but do not perform any streaming." ;
+      Arg.Unit (fun () ->
+                  secondary_task := true ;
+                  dont_run := true),
+      "Check and evaluate scripts but do not perform any streaming." ;
+
+      ["-p";"--parse-only"],
+      Arg.Unit (fun () ->
+                  secondary_task := true ;
+                  parse_only := true),
+      "Parse scripts but do not type-check and run them." ;
 
       ["-q";"--quiet"],
       Arg.Unit (fun () -> Log.conf_stdout#set false),
@@ -319,16 +337,19 @@ let options =
                   secondary_task := true ;
                   check_pervasives () ;
                   Doc.print_xml (Plug.plugs:Doc.item)),
-      Printf.sprintf "List all %s, output as XML."
-        (String.concat ", " (Plug.list ())) ;
+      Printf.sprintf
+        "List all plugins (builtin scripting values, \
+         supported formats and protocols), \
+         output as XML." ;
 
       ["--list-plugins"],
       Arg.Unit (fun () ->
                   secondary_task := true ;
                   check_pervasives () ;
                   Doc.print (Plug.plugs:Doc.item)),
-      Printf.sprintf "List all %s."
-        (String.concat ", " (Plug.list ())) ;
+      Printf.sprintf
+        "List all plugins (builtin scripting values, \
+         supported formats and protocols)." ;
 
       ["--no-pervasives"],
       Arg.Clear pervasives,
@@ -342,15 +363,16 @@ let options =
 
       ["--version"],
       Arg.Unit (fun () ->
-                  Printf.printf "Liquidsoap %s%s.\n%s\n%s\n"
-                    Configure.version SVN.rev
-                    "Copyright (c) 2003-2007 Savonet team"
-                    ("Liquidsoap is open-source software, "^
-                     "released under GNU General Public License.\n"^
-                     "See <http://savonet.sf.net> for more information.") ;
+                  Printf.printf
+                    "Liquidsoap %s%s.\n\
+                     Copyright (c) 2003-2008 Savonet team\n\
+                     Liquidsoap is open-source software, \
+                     released under GNU General Public License.\n\
+                     See <http://savonet.sf.net> for more information.\n"
+                    Configure.version SVN.rev ;
                   exit 0),
      "Display liquidsoap's version." ;
-    
+
     ["--"],
     Arg.Unit (fun () -> Arg.current := Array.length Shebang.argv - 1),
     "Stop parsing the command-line and pass subsequent items to the script." ]
@@ -405,8 +427,8 @@ let check_directories () =
         let routes = Configure.conf#routes conf_path#ut in
           Printf.printf
             "To change it, add the following to your script:\n";
-          Printf.printf "  set %s = \"<path>\"\n"
-            (Conf.string_of_path	(List.hd routes)) ;
+          Printf.printf "  set(%S, \"<path>\")\n"
+            (Conf.string_of_path (List.hd routes)) ;
           exit 1
       end
   in
@@ -434,6 +456,7 @@ let () =
       Tutils.main ()
     in
       ignore (Init.at_stop cleanup) ;
+      ignore (Init.at_stop (fun () -> Duppy.stop Tutils.scheduler)) ;
       if Source.has_outputs () then
         if not !dont_run then begin
           check_directories () ;
@@ -443,5 +466,5 @@ let () =
       else
         (* If there's no output and no secondary task has been performed,
          * warn the user that his scripts didn't define any output. *)
-        if not (!dont_run || !secondary_task) then
+        if not !secondary_task then
           Printf.printf "No output defined, nothing to do.\n"
