@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -126,13 +126,13 @@ let register_add =
 
   Lang.add_operator "add"
     ~category:Lang.SoundProcessing
-    ~descr:("Mix sources, with optional normalization. " ^
-            "Only relay metadata from the first source that is effectively " ^
-            "summed.")
+    ~descr:"Mix sources, with optional normalization. \
+           Only relay metadata from the first source that is effectively \
+           summed."
     [ "normalize", Lang.bool_t, Some (Lang.bool true), None ;
       "weights", Lang.list_t Lang.int_t, Some (Lang.list []),
-      Some ("Relative weight of the sources in the sum. " ^
-            "The empty list stands for the homogeneous distribution.") ;
+      Some "Relative weight of the sources in the sum. \
+            The empty list stands for the homogeneous distribution." ;
       "", Lang.list_t Lang.source_t, None, None ]
     (fun p ->
        let sources = Lang.to_source_list (List.assoc "" p) in
