@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -38,7 +38,8 @@ module Generated = Generated.From_Float_pcm_Generator
  * This operator is data-dependent only because it uses Generator, which is
  * specialized for audio frames. Otherwise it should be generic. That's why I
  * stick to the Frame module here, and ticks units. *)
-class cross s ~cross_length ~inhibit ~minimum_length f =
+class cross s ?(meta="liq_start_next") ~cross_length
+              ~conservative ~inhibit ~minimum_length f =
 object (self)
   inherit operator [s] as super
 
@@ -66,7 +67,7 @@ object (self)
   val mutable cur_cross_length = None
 
   (* It is tricky to get the cross_length.
-   * The metadata 'liq_start_next' can come through the transition function,
+   * The metadata meta can come through the transition function,
    * from which other metadata comes.
    * Our solution is to scan every metadata packet until we found a value,
    * and reset the value when a transition starts.
@@ -79,7 +80,7 @@ object (self)
           List.iter
             (fun (p,m) ->
                if p>=pos then
-                 match Utils.hashtbl_get m "liq_start_next" with
+                 match Utils.hashtbl_get m meta with
                    | None -> ()
                    | Some v ->
                        try
@@ -103,7 +104,7 @@ object (self)
           (* TODO 0 is often answered just before the beginning of a track
            * I'd prefer -1 if possible. Otherwise add more info in the state.
            * Or simply don't bother about that harmless transition. *)
-          let rem = source#remaining in
+          let rem = if conservative then 0 else source#remaining in
             if rem < 0 || rem > cross_length then begin
               source#get ab ;
               self#update_cross_length ab p
@@ -160,6 +161,7 @@ object (self)
        * We compose the end of a track with the original source [s] instead of
        * the composed [source]. *)
       let s =
+        if not s#is_ready then self#log#f 3 "No ready track yet." ;
         if Generator.length buffer > minimum_length then
           Lang.to_source
             (Lang.apply f ["",Lang.source (new Generated.consumer buffer);
@@ -195,7 +197,7 @@ object (self)
       | _ -> source#abort_track
 
   method after_output =
-    self#clear_cache ;
+    self#advance ;
     match status with
       | `Stopped (s,_) -> s#after_output
       | _ -> source#after_output
@@ -206,23 +208,33 @@ let () =
   Lang.add_operator "cross"
     [ "duration", Lang.float_t, Some (Lang.float 5.),
       Some
-        ("Duration in seconds of the crossed end of track. "^
-         "This value can be set on a per-file basis using the metadata field "^
-         "'liq_start_next.") ;
+        "Duration in seconds of the crossed end of track. \
+         This value can be changed on a per-file basis using \
+         a special metadata field." ;
+
+      "override", Lang.string_t, Some (Lang.string "liq_start_next"),
+      Some "Metadata field which, if present and containing a float, \
+            overrides the 'duration' parameter for current track." ;
 
       "inhibit", Lang.float_t, Some (Lang.float (-1.)),
-      Some ("Minimum delay between two transitions. It is useful in order to "^
-            "avoid that a transition is triggered on top of another when "^
-            "an end-of-track occurs in the first one. Negative values mean "^
-            "'same as duration'.") ;
+      Some  "Minimum delay between two transitions. It is useful in order to \
+             avoid that a transition is triggered on top of another when \
+             an end-of-track occurs in the first one. Negative values mean \
+             <code>duration+1</code>. \
+             Warning: zero inhibition can cause infinite loops." ;
 
       "minimum", Lang.float_t, (Some (Lang.float (-1.))),
-      Some ("Minimum duration (in sec.) for a cross: "^
-            "If the track ends without any warning (e.g. in case of skip) "^
-            "there may not be enough data for a decent composition. "^
-            "Set to 0. to avoid having transitions after skips, "^
-            "or more to avoid transitions on short tracks. "^
-            "With the negative default, transitions always occur.") ;
+      Some "Minimum duration (in sec.) for a cross: \
+            If the track ends without any warning (e.g. in case of skip) \
+            there may not be enough data for a decent composition. \
+            Set to 0. to avoid having transitions after skips, \
+            or more to avoid transitions on short tracks. \
+            With the negative default, transitions always occur." ;
+
+      "conservative", Lang.bool_t, Some (Lang.bool false),
+      Some "Do not trust remaining time estimations, always buffering \
+            data in advance. This avoids being tricked by skips, either \
+            manual or caused by skip_blank()." ;
 
       "",
       Lang.fun_t [false,"",Lang.source_t;false,"",Lang.source_t] Lang.source_t,
@@ -239,14 +251,17 @@ let () =
     (fun p ->
        let duration = Lang.to_float (List.assoc "duration" p) in
        let cross_length = Fmt.ticks_of_seconds duration in
-
+       let meta = Lang.to_string (List.assoc "override" p) in
        let minimum = Lang.to_float (List.assoc "minimum" p) in
        let minimum_length = Fmt.ticks_of_seconds minimum in
 
        let inhibit = Lang.to_float (List.assoc "inhibit" p) in
-       let inhibit = if inhibit < 0. then duration else inhibit in
+       let inhibit = if inhibit < 0. then duration +. 1. else inhibit in
        let inhibit = Fmt.ticks_of_seconds inhibit in
 
+       let conservative = Lang.to_bool (List.assoc "conservative" p) in
+
        let f = Lang.assoc "" 1 p in
        let source = Lang.to_source (Lang.assoc "" 2 p) in
-         ((new cross source ~cross_length ~inhibit ~minimum_length f):>source))
+         ((new cross source ~meta ~cross_length
+             ~inhibit ~conservative ~minimum_length f):>source))
