@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ open Source
 (** Fade-in at the beginning of every frame.
   * The [duration] is in seconds.
   * If the initial flag is set, only the first/current track is faded in. *)
-class fade_in ?(initial=false) duration fader source =
+class fade_in ?(meta="liq_fade_in") ?(initial=false) duration fader source =
 object (self)
 
   inherit operator [source] as super
@@ -48,7 +48,7 @@ object (self)
               match AFrame.get_metadata ab p1 with
                 | None -> duration
                 | Some m ->
-                    match Utils.hashtbl_get m "liq_fade_in" with
+                    match Utils.hashtbl_get m meta with
                       | Some d -> (try float_of_string d with _ -> duration)
                       | None -> duration
             in
@@ -76,7 +76,7 @@ end
 (** Fade-out after every frame.
   * If the final flag is set, the fade-out happens as of instantiation
   * and the source becomes unavailable once it's finished. *)
-class fade_out ?(final=false) duration fader source =
+class fade_out ?(meta="liq_fade_out") ?(final=false) duration fader source =
 object (self)
 
   inherit operator [source] as super
@@ -93,10 +93,10 @@ object (self)
 
   method remaining =
     if final then Fmt.ticks_of_samples remaining else source#remaining
-  method is_ready = remaining > 0 && source#is_ready
+  method is_ready = (remaining > 0 || not final) && source#is_ready
 
   method get_frame ab =
-    if remaining <= 0 then
+    if final && remaining <= 0 then
       (* This happens in final mode at the end of the remaining time. *)
       Frame.add_break ab (Frame.position ab)
     else
@@ -113,7 +113,7 @@ object (self)
                 match AFrame.get_metadata ab offset1 with
                   | None -> duration
                   | Some m ->
-                      match Utils.hashtbl_get m "liq_fade_out" with
+                      match Utils.hashtbl_get m meta with
                         | None -> duration
                         | Some d -> (try float_of_string d with _ -> duration)
               in
@@ -146,10 +146,13 @@ end
 (** Lang interface *)
 
 let proto =
-  [ "duration", Lang.float_t, Some (Lang.float 3.), None ;
+  [ "duration", Lang.float_t, Some (Lang.float 3.), 
+     Some "Duration of the fading. \
+           This value can be set on a per-file basis using the metadata field \
+           passed as override." ;
     "type", Lang.string_t, Some (Lang.string "lin"),
-    Some ("fader shape (lin|sin|log|exp): "^
-          "linear, sinusoidal, logarithmic or exponential") ;
+    Some "Fader shape (lin|sin|log|exp): \
+          linear, sinusoidal, logarithmic or exponential." ;
     "", Lang.source_t, None, None ]
 
 let extract p =
@@ -186,27 +189,37 @@ let extract p =
   ),
   Lang.to_source (List.assoc "" p)
 
+let override_doc = 
+  Some "Metadata field which, if present and containing a float, \
+        overrides the 'duration' parameter for current track." 
+
 let () =
-  Lang.add_operator "fade.in" proto
+  Lang.add_operator 
+    "fade.in" (("override", Lang.string_t, Some (Lang.string "liq_fade_in"),
+               override_doc) :: proto)
     ~category:Lang.SoundProcessing
     ~descr:("Fade the beginning of tracks. Metadata 'liq_fade_in' can be used "^
             "to set the duration for a specific track (float in seconds).")
     (fun p ->
        let d,f,s = extract p in
-         ((new fade_in d f s):>source)) ;
+       let meta = Lang.to_string (List.assoc "override" p) in
+         ((new fade_in ~meta d f s):>source)) ;
   Lang.add_operator "fade.initial" proto
     ~category:Lang.SoundProcessing
     ~descr:"Fade the beginning of a stream."
     (fun p ->
        let d,f,s = extract p in
          ((new fade_in ~initial:true d f s):>source)) ;
-  Lang.add_operator "fade.out" proto
+  Lang.add_operator 
+    "fade.out" (("override", Lang.string_t, Some (Lang.string "liq_fade_out"),
+               override_doc) :: proto) 
     ~category:Lang.SoundProcessing
     ~descr:("Fade the end of tracks. Metadata 'liq_fade_out' can be used to "^
             "set the duration for a specific track (float in seconds).")
     (fun p ->
        let d,f,s = extract p in
-         ((new fade_out d f s):>source)) ;
+       let meta = Lang.to_string (List.assoc "override" p) in
+         ((new fade_out ~meta d f s):>source)) ;
   Lang.add_operator "fade.final" proto
     ~category:Lang.SoundProcessing
     ~descr:"Fade a stream to silence."
