@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -76,8 +76,11 @@ let () =
   Lang.add_operator "filter"
     [ "freq", Lang.float_getter_t 1, None, None ;
       "q", Lang.float_getter_t 2, Some (Lang.float 1.), None ;
-      "mode", Lang.string_t, None, Some "low|high|band|notch" ;
-      "wetness", Lang.float_getter_t 3, Some (Lang.float 1.), Some "How much of the original signal should be added (1. means only filtered and 0. means only original signal).";
+      "mode", Lang.string_t, None,
+      Some "Available modes are 'low', 'high', 'band' and 'notch'." ;
+      "wetness", Lang.float_getter_t 3, Some (Lang.float 1.),
+      Some "How much of the original signal should be added \
+            (1. means only filtered and 0. means only original signal).";
       "", Lang.source_t, None, None ]
     ~category:Lang.SoundProcessing
     ~descr:"Perform several kinds of filtering on the signal"
