@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -501,7 +501,6 @@ let () =
          Lang.to_source (f "") in
          ((new iir src Butterworth Band_stop order freq1 freq2 0. debug):>source)
     ) ;
-    
 
   Lang.add_operator "filter.iir.resonator.bandpass"
     [
@@ -523,6 +522,46 @@ let () =
          ((new iir src Resonator Band_pass 0 freq 0. q debug):>source)
     ) ;
 
+  Lang.add_operator "filter.iir.resonator.bandstop"
+    [
+      "frequency", Lang.float_t, None, Some "Corner frequency" ;
+      "q", Lang.float_t, Some (Lang.float 60.), Some "Quality factor" ;
+(*      "order", Lang.int_t, Some (Lang.int 4), Some "Filter order" ;*)
+      "debug", Lang.bool_t, Some (Lang.bool false), Some "Debug output" ;
+      "", Lang.source_t, None, None
+    ]
+    ~category:Lang.SoundProcessing
+    ~descr:"IIR filter"
+    (fun p ->
+       let f v = List.assoc v p in
+       let freq, q, debug, src =
+         Lang.to_float (f "frequency"),
+         Lang.to_float (f "q"),
+         Lang.to_bool (f "debug"),
+         Lang.to_source (f "") in
+         ((new iir src Resonator Band_pass 0 freq 0. q debug):>source)
+    ) ;
+
+  Lang.add_operator "filter.iir.resonator.allpass"
+    [
+      "frequency", Lang.float_t, None, Some "Corner frequency" ;
+      "q", Lang.float_t, Some (Lang.float 60.), Some "Quality factor" ;
+(*      "order", Lang.int_t, Some (Lang.int 4), Some "Filter order" ;*)
+      "debug", Lang.bool_t, Some (Lang.bool false), Some "Debug output" ;
+      "", Lang.source_t, None, None
+    ]
+    ~category:Lang.SoundProcessing
+    ~descr:"IIR filter"
+    (fun p ->
+       let f v = List.assoc v p in
+       let freq, q, debug, src =
+         Lang.to_float (f "frequency"),
+         Lang.to_float (f "q"),
+         Lang.to_bool (f "debug"),
+         Lang.to_source (f "") in
+         ((new iir src Resonator Band_pass 0 freq 0. q debug):>source)
+    ) ;
+
 (*  Lang.add_operator "filter.crossover"
     [
       "bands", Lang.list_t Lang.float_t,
