@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -64,7 +64,10 @@ object (self)
       done ;
       AFrame.add_break buf (AFrame.size buf)
 
-  method abort_track = ()
+  method abort_track =
+    for i = 0 to n-1 do
+      if sel.(i) then source.(i)#abort_track
+    done
 
   method status i =
     Printf.sprintf
@@ -85,8 +88,10 @@ object (self)
     Server.add ~ns "skip"
       (fun a ->
          source.(int_of_string a)#abort_track ;
-         "OK") ;
-    Server.add ~ns "volume"
+         "OK") 
+	       ~descr:"Skip current track on all enabled sources.";
+    Server.add ~ns "volume" ~descr:"Set volume for a given source." 
+               ~usage:"volume <source nb> <vol%>"
       (fun a ->
          if Str.string_match (Str.regexp "\\([0-9]+\\) \\([0-9]+\\)") a 0 then
            let i = int_of_string (Str.matched_group 1 a) in
@@ -94,8 +99,9 @@ object (self)
              vol.(i) <- (float v)/.100. ;
              self#status i
          else
-           "Usage: vol [source nb] [vol%]") ;
-    Server.add ~ns "select"
+           "Usage: volume <source nb> <vol%>") ;
+    Server.add ~ns "select" ~descr:"Enable/disable a source."
+               ~usage:"select <source nb> <true|false>"
       (fun a ->
          if Str.string_match
               (Str.regexp "\\([0-9]+\\) \\(true\\|false\\)") a 0 then
@@ -104,8 +110,9 @@ object (self)
              sel.(i) <- v = "true" ;
              self#status i
          else
-           "Usage: select [source nb] [true|false]") ;
-    Server.add ~ns "single"
+           "Usage: select <source nb> <true|false>") ;
+    Server.add ~ns "single" ~descr:"Enable/disable automatic stop at the end of track."
+               ~usage:"single <source nb> <true|false>"
       (fun a ->
          if Str.string_match
               (Str.regexp "\\([0-9]+\\) \\(true\\|false\\)") a 0 then
@@ -114,10 +121,10 @@ object (self)
              single.(i) <- v = "true" ;
              self#status i
          else
-           "Usage: single [source nb] [true|false]") ;
-    Server.add ~ns "status"
+           "Usage: single <source nb> <true|false>") ;
+    Server.add ~ns "status" ~descr:"Display current status."
       (fun a -> self#status (int_of_string a)) ;
-    Server.add ~ns "inputs"
+    Server.add ~ns "inputs" ~descr:"Print the list of input sources."
       (fun _ -> Array.fold_left (fun e s -> e^" "^s#id) "" source)
 
 end
