@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,27 +24,50 @@ open Source
 
 (** In this file [length]s are in samples, [threshold] are RMS (in [0.;1.]). *)
 
-class on_blank ~length ~threshold handler source =
-object (self)
-  inherit operator [source]
+
+class virtual base ~length ~threshold = 
+object(self)
 
   val mutable blank_len = 0
-  val mutable in_blank = false
 
-  method stype = source#stype
-  method is_ready = source#is_ready
-  method abort_track = source#abort_track ; blank_len <- 0
-  method remaining = source#remaining
+  method abort_track =
+    blank_len <- 0
 
-  method check_blank s p0 =
+  method virtual on_noise : unit
+
+  method check_blank s p0 = 
     let len = AFrame.position s - p0 in
     let rms = Float_pcm.rms (AFrame.get_float_pcm s) p0 len in
     let noise = ref false in
       Array.iter (fun r -> if r > threshold then noise := true) rms;
       if !noise then
-        blank_len <- 0
+        begin
+          blank_len <- 0 ;
+          self#on_noise
+        end
       else
-        blank_len <- blank_len + len
+        if blank_len + len <= length then
+          blank_len <- blank_len + len
+        else
+          blank_len <- length + 1
+
+end
+
+class on_blank ~length ~threshold handler source =
+object (self)
+  inherit operator [source]
+  inherit base ~length ~threshold as base
+
+  val mutable in_blank = false
+
+  method stype = source#stype
+  method is_ready = source#is_ready
+  method abort_track = 
+    source#abort_track ; 
+    base#abort_track
+  method remaining = source#remaining
+
+  method on_noise = ()
 
   method get_frame ab =
     let p0 = AFrame.position ab in
@@ -56,7 +79,7 @@ object (self)
         else
           if not in_blank then begin
             in_blank <- true ;
-            ignore (Lang.eval (Lang.apply handler []))
+            ignore (Lang.apply handler [])
           end
       end
 end
@@ -66,23 +89,16 @@ let log = Dtools.Log.make ["noblank"]
 class skip ~length ~threshold source =
 object (self)
   inherit operator [source]
-
-  val mutable blank_len = 0
+  inherit base ~length ~threshold as base
 
   method stype = source#stype
   method is_ready = source#is_ready
-  method abort_track = source#abort_track ; blank_len <- 0
+  method abort_track = 
+    source#abort_track ; 
+    base#abort_track
   method remaining = source#remaining
 
-  method check_blank s p0 =
-    let len = AFrame.position s - p0 in
-    let rms = Float_pcm.rms (AFrame.get_float_pcm s) p0 len in
-    let noise = ref false in
-      Array.iter (fun r -> if r > threshold then noise := true) rms;
-      if !noise then
-        blank_len <- 0
-      else
-        blank_len <- blank_len + len
+  method on_noise = ()
 
   method get_frame ab =
     let p0 = AFrame.position ab in
@@ -99,8 +115,8 @@ end
 class strip ~length ~threshold source =
 object (self)
   inherit active_operator source
+  inherit base ~length ~threshold as base
 
-  val mutable blank_len = 0
   val mutable stripping = false
 
   method stype = Fallible
@@ -108,18 +124,10 @@ object (self)
   method remaining = if stripping then 0 else source#remaining
   method abort_track =
     source#abort_track ;
-    blank_len <- 0 ;
+    base#abort_track ;
     stripping <- false
 
-  method check_blank s p0 =
-    let len = AFrame.position s - p0 in
-    let rms = Float_pcm.rms (AFrame.get_float_pcm s) p0 len in
-    let noise = ref false in
-      Array.iter (fun r -> if r > threshold then noise := true) rms;
-      if !noise then
-        ( blank_len <- 0 ; stripping <- false )
-      else
-        blank_len <- blank_len + len
+  method on_noise = stripping <- false
 
   method get_frame ab =
     let p0 = AFrame.position ab in
@@ -146,8 +154,8 @@ end
 class eat ~at_beginning ~length ~threshold source =
 object (self)
   inherit operator [source] as super
+  inherit base ~length ~threshold as base
 
-  val mutable blank_len = 0
   val mutable stripping = false
   val mutable beginning = true
 
@@ -156,23 +164,13 @@ object (self)
   method remaining = source#remaining
   method abort_track =
     source#abort_track ;
-    blank_len <- 0 ;
+    base#abort_track ;
     stripping <- false ;
     beginning <- true
 
-  method check_blank s p0 =
-    let len = AFrame.position s - p0 in
-    let rms = Float_pcm.rms (AFrame.get_float_pcm s) p0 len in
-    let noise = ref false in
-      Array.iter (fun r -> if r > threshold then noise := true) rms;
-      if !noise then
-        (
-          blank_len <- 0 ;
-          stripping <- false ;
-          beginning <- false
-        )
-      else
-        blank_len <- blank_len + len
+  method on_noise = 
+    stripping <- false ;
+    beginning <- false
 
   method get_frame ab =
     let first = ref true in
