@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ object (self)
         (fun (i,m) ->
            if i>=p then begin
              self#log#f 4 "Got metadata at position %d: calling handler..." i ;
-             ignore (Lang.eval (Lang.apply f ["",Lang.metadata m]))
+             ignore (Lang.apply f ["",Lang.metadata m])
            end)
         (Frame.get_all_metadata ab)
 
