@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ object (self)
     if ns = [] then ns <- Server.register [self#id] "store_metadata" ;
     self#set_id (Server.to_string ns) ;
     super#wake_up activation ;
-    Server.add ~ns "get"
+    Server.add ~ns "get" ~descr:"Print the stored metadata."
       (fun _ ->
          let q = metadata_q in
            (fst (Queue.fold
