@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -33,7 +33,12 @@ type transition = Lang.value
 
 type child = {
   source     : source ;
-  transition : transition
+  transition : transition ;
+  (* Also remind the current metadata in order to be able to restore it
+   * when switching back in the middle of a track. This is an expected
+   * behaviour, but notice that it makes extra assumptions about the nature of
+   * metadata, tying them to the notion of track. *)
+  mutable cur_meta : Request.metadata option ;
 }
 
 (** The switch can either happen at any time in the stream (insensitive)
@@ -42,16 +47,21 @@ type child = {
   * anticipation parameter specified as a number of ticks. *)
 type track_mode = Sensitive of int | Insensitive
 
-class virtual switch ?(mode=Sensitive 0) (cases : child list) =
+class virtual
+  switch ?(mode=Sensitive 0) ?(replay_meta=true) (cases : child list) =
 object (self)
 
   inherit operator (List.map (fun x -> x.source) cases) as super
 
   val mutable selected = None
+
+  (** We have to explictly manage our children as they are dynamically created
+    * by application of the transition functions. In particular we need a list
+    * of all children that have output data in the current round. *)
   val mutable to_finish = []
 
   (** Indicates that the former child was left without having finished its
-    * track. *)
+    * track, in which case the switch will artifically produce an EOT. *)
   val mutable need_eot = false
 
   (** The selection method should return None or Some c,
@@ -71,7 +81,7 @@ object (self)
             c
 
   method after_output =
-    self#clear_cache ;
+    self#advance ;
     List.iter (fun s -> s#after_output) to_finish ;
     to_finish <- [] ;
     List.iter (fun s -> s.source#after_output) cases ;
@@ -104,7 +114,9 @@ object (self)
 
   method get_frame ab =
     (* Choose the next child to be played.
-     * [forget] tells that the current child has finished its track. *)
+     * [forget] tells that the current child has finished its track,
+     * in which case a transition does not make sense and would actually start
+     * playing a next track (if available) on the left child. *)
     let reselect ?(forget=false) () =
       if not forget then need_eot <- true ;
       match let c = self#cached_select in cached_selected <- None ; c with
@@ -119,17 +131,31 @@ object (self)
                 selected <- Some (c,c.source)
             | Some (old_c,old_s) when old_c != c ->
                 self#log#f 3 "switch to %s with%s transition"
-                     c.source#id
-                     (if forget then " forgetful" else "") ;
+                  c.source#id
+                  (if forget then " forgetful" else "") ;
                 old_s#leave (self:>source) ;
                 to_finish <- old_s::to_finish ;
                 let old_source =
                   if forget then Blank.empty else old_c.source
                 in
+                let new_source =
+                  (* Force insertion of old metadata if relevant.
+                   * It can't be done in a static way: we need to start
+                   * pulling data to see if new metadata comes out, in case
+                   * the source was shared and kept streaming from somewhere
+                   * else (this is thanks to Frame.get_chunk).
+                   * A quicker hack might have been doable if there wasn't a
+                   * transition in between. *)
+                  match c.cur_meta with
+                    | Some m when replay_meta ->
+                        new Insert_metadata.replay m c.source
+                    | _ -> c.source
+                in
                 let s =
                   Lang.to_source
                     (Lang.apply c.transition
-                       [ "",Lang.source old_source ; "",Lang.source c.source ])
+                       [ "",Lang.source old_source ;
+                         "",Lang.source new_source ])
                 in
                   s#get_ready activation ;
                   selected <- Some (c,s)
@@ -160,8 +186,25 @@ object (self)
           (* Our #is_ready, and caching, ensure the following. *)
           assert (selected <> None) ;
           self#get_frame ab
-      | Some (_,s) ->
+      | Some (c,s) ->
           s#get ab ;
+          c.cur_meta <-
+            (if Frame.is_partial ab then
+               None
+             else
+               match
+                 List.fold_left
+                   (function
+                      | None -> fun (p,m) -> Some (p,m)
+                      | Some (curp,curm) -> fun (p,m) ->
+                          Some (if p>=curp then (p,m) else (curp,curm)))
+                   (match c.cur_meta with
+                      | None -> None
+                      | Some m -> Some (-1,m))
+                   (Frame.get_all_metadata ab)
+               with
+                 | None -> None
+                 | Some (_,m) -> Some m) ;
           if Frame.is_partial ab then
             reselect ~forget:true ()
           else
@@ -190,21 +233,25 @@ let common = [
   "track_sensitive", Lang.bool_t, Some (Lang.bool true),
   Some "Re-select only on end of tracks." ;
 
+  "replay_metadata", Lang.bool_t, Some (Lang.bool true),
+  Some "Replay the last metadata of a child when switching to it \
+        in the middle of a track." ;
+
   "before", Lang.float_t, Some (Lang.float 0.),
-  Some ("EXPERIMENTAL: for track_sensitive switches, "^
-        "trigger transitions before the end of track.") ;
+  Some "EXPERIMENTAL: for track_sensitive switches, \
+        trigger transitions before the end of track." ;
 
   "transitions",
   Lang.list_t
     (Lang.fun_t [ false,"",Lang.source_t ;
                   false,"",Lang.source_t ] Lang.source_t),
   Some (Lang.list []),
-  Some "Transition functions, padded with (fun (x,y) -> y) functions."
+  Some "Transition functions, \
+        padded with <code>fun (x,y) -> y</code> functions."
 ]
 
 let default_transition =
-  Lang.val_fun [ "","x",Lang.source_t,None ;
-                 "","y",Lang.source_t,None ] (Lang.var "y")
+  Lang.val_fun [ "","x",None ; "","y",None ] (fun e -> List.assoc "y" e)
 
 let extract_common p l =
   let ts =
@@ -225,21 +272,24 @@ let extract_common p l =
       else
         tr
   in
-    ts,tr
+  let replay = Lang.to_bool (List.assoc "replay_metadata" p) in
+    replay,ts,tr
 
 (** Switch: switch according to user-defined predicates. *)
 
 let satisfied f = Lang.to_bool (Lang.apply f [])
 let trivially_true = function
-  | { Lang.value = Lang.Fun (_, { Lang.value = Lang.Bool true }) } -> true
+  | { Lang.value =
+        Lang.Fun (_,_,_,{ Lang_values.term = Lang_values.Bool true }) } -> true
   | _ -> false
 
 let third (_,_,s) = s
 
-class lang_switch mode (children : (Lang.value * bool * child) list) =
+class
+  lang_switch mode ?replay_meta (children : (Lang.value * bool * child) list) =
 object (self)
 
-  inherit switch ~mode (List.map third children)
+  inherit switch ~mode ?replay_meta (List.map third children)
 
   method private select =
     let selected s =
@@ -274,8 +324,8 @@ let () =
   let pred_t = Lang.fun_t [] Lang.bool_t in
   let proto = 
     [ "single", Lang.list_t Lang.bool_t, Some (Lang.list []),
-      Some ("Forbid the selection of a branch for two tracks in a row. "^
-            "The empty list stands for [false,...,false].") ;
+      Some "Forbid the selection of a branch for two tracks in a row. \
+            The empty list stands for <code>[false,...,false]</code>." ;
       "", Lang.list_t (Lang.product_t pred_t Lang.source_t), None,
       Some "Sources with the predicate telling when they can be played." ]
   in
@@ -292,7 +342,7 @@ let () =
                   pred, Lang.to_source s)
            (Lang.to_list (List.assoc "" p))
          in
-         let ts,tr = extract_common p (List.length children) in
+         let replay_meta,ts,tr = extract_common p (List.length children) in
          let singles =
            List.map Lang.to_bool (Lang.to_list (List.assoc "single" p))
          in
@@ -303,7 +353,9 @@ let () =
              singles
          in
          let children =
-           List.map2 (fun t (f,s) -> f,{ source = s ; transition = t })
+           List.map2
+             (fun t (f,s) ->
+                f,{ source = s ; cur_meta = None ; transition = t })
              tr children
          in
          let children =
@@ -313,13 +365,13 @@ let () =
                           (List.assoc "single" p,
                            "there should be exactly one flag per children"))
          in
-           ((new lang_switch ts children):>source))
+           ((new lang_switch ~replay_meta ts children):>source))
 
 (** Fallback selector: switch to the first ready source. *)
-class fallback mode children =
+class fallback ?replay_meta mode children =
 object
 
-  inherit switch ~mode children
+  inherit switch ~mode ?replay_meta children
 
   method private select =
     try Some (List.find (fun s -> s.source#is_ready) children) with
@@ -345,16 +397,19 @@ let () =
       (common@proto)
       (fun p ->
          let children = Lang.to_source_list (List.assoc "" p) in
-         let ts,tr = extract_common p (List.length children) in
-         let children = List.map2 (fun t s -> { transition = t ;
-                                                source = s }) tr children in
-           ((new fallback ts children):>source))
+         let replay_meta,ts,tr = extract_common p (List.length children) in
+         let children =
+           List.map2
+             (fun t s -> { transition = t ; cur_meta = None ; source = s })
+             tr children
+         in
+           ((new fallback ~replay_meta ts children):>source))
 
 (** Random switch *)
 exception Found of child
-class random strict mode children =
+class random ?replay_meta strict mode children =
 object
-  inherit switch ~mode (List.map snd children)
+  inherit switch ?replay_meta ~mode (List.map snd children)
 
   val mutable pos = -1
 
@@ -397,7 +452,7 @@ let () =
       ~descr:"At the beginning of every track, select a random ready child."
       (fun p ->
          let children = Lang.to_source_list (List.assoc "" p) in
-         let ts,tr = extract_common p (List.length children) in
+         let replay_meta,ts,tr = extract_common p (List.length children) in
          let weights =
            List.map Lang.to_int (Lang.to_list (List.assoc "weights" p))
          in
@@ -407,10 +462,16 @@ let () =
          in
          let strict = Lang.to_bool (List.assoc "strict" p) in
          let children =
+           if List.length weights <> List.length children then
+             raise
+               (Lang.Invalid_value
+                  ((List.assoc "weights" p),
+                   "there should be as many weights as sources")) ;
            List.map2
              (fun w c -> w,c)
              weights
-             (List.map2 (fun tr s -> { transition = tr ; source = s })
+             (List.map2
+                (fun tr s -> { transition = tr ; source = s ; cur_meta = None })
                 tr children)
          in
-           ((new random strict ts children):>source)) ;
+           ((new random ~replay_meta strict ts children):>source)) ;
