@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -33,16 +33,16 @@ let proto =
   [ "start", Lang.bool_t, Some (Lang.bool true),
     Some "Start output threads on operator initialization." ;
     "restart", Lang.bool_t, Some (Lang.bool false),
-    Some
-      ("Restart output after a failure. By default, liquidsoap will stop " ^
-       "if the output failed.") ;
+    Some "Restart output after a failure. By default, liquidsoap will stop \
+          if the output failed." ;
     "restart_delay", Lang.int_t, Some (Lang.int 3),
-    Some
-      ("Delay, in seconds, before attempting new connection, if restart " ^
-       "is enabled.") ;
+    Some "Delay, in seconds, before attempting new connection, if restart \
+          is enabled." ;
     "host", Lang.string_t, Some (Lang.string "localhost"), None ;
     "port", Lang.int_t, Some (Lang.int 8000), None ;
-    "user", Lang.string_t, Some (Lang.string "source"), None ;
+    "user", Lang.string_t, Some (Lang.string "source"),
+    Some "User for shout source connection. \
+          Useful only in special cases, like with per-mountpoint users." ;
     "password", Lang.string_t, Some (Lang.string "hackme"), None ;
     "genre", Lang.string_t, Some (Lang.string "Misc"), None ;
     "url", Lang.string_t, Some (Lang.string "http://savonet.sf.net"), None ;
@@ -51,7 +51,9 @@ let proto =
     "public", Lang.bool_t, Some (Lang.bool true), None ;
     "multicast_ip", Lang.string_t, Some (Lang.string no_multicast), None ;
     "sync", Lang.bool_t, Some (Lang.bool false),
-    Some "let shout do the synchronization"
+    Some "Let shout do the synchronization.";
+    "dumpfile", Lang.string_t, Some (Lang.string ""), 
+    Some "Dump stream to file, for debugging purpose. Disabled if empty."
   ]
 
 (** Abstract class for sending encoded data to an shout-compatible server,
@@ -75,6 +77,11 @@ class virtual ['a] output
     let password = s "password" in
     let genre = s "genre" in
     let url = s "url" in
+    let dumpfile = 
+      match s "dumpfile" with
+        | "" -> None
+	| s -> Some s
+    in
     let description = s "description" in
     let public = e Lang.to_bool "public" in
     let multicast_ip = s "multicast_ip" in
@@ -87,6 +94,7 @@ object (self)
 
   val mutable connection = None
   val mutable last_attempt = 0.
+  val mutable dump = None
 
   initializer
     if sync then
@@ -106,23 +114,25 @@ object (self)
              *      is that the encoder gets restarted every delay seconds,
              *      even when shout doesn't actually get restarted. *)
             self#output_stop ;
-            self#output_start ;
-            self#send b
+            self#output_start
           end
       | Some c ->
           (* TODO think about some limitation of shout restarting *)
           begin try
             if sync then Shout.sync c ;
-            Shout.send c b
+            Shout.send c b ;
+            match dump with
+              | Some s -> output_string s b
+              | None -> () 
           with
             | Shout.Socket ->
-                self#log#f 2 "%s%s %.f sec"
-                  "Shout socket error: timeout, network failure, "
-                  "server shutdown? Restarting the output in "
+                self#log#f 2
+                  "Shout socket error: timeout, network failure, \
+                   server shutdown? Restarting the output in %.f seconds."
                   restart_delay  ;
-                  (* Ask for a restart after last_attempt. *)
-                  self#output_stop ;
-                  last_attempt <- Unix.time ()
+                (* Ask for a restart after last_attempt. *)
+                self#output_stop ;
+                last_attempt <- Unix.time ()
             end
 
   method output_stop =
@@ -130,12 +140,21 @@ object (self)
       | None -> ()
       | Some c ->
           begin try Shout.close c with _ -> () end ;
-          connection <- None
+          connection <- None ;
+          match dump with
+            | Some f -> close_out f
+            | None -> ()
 
   method output_start =
     assert (connection = None) ;
     let conn = new_shout () in
 
+    begin 
+      match dumpfile with
+        | Some f -> dump <- Some (open_out_bin f)
+        | None -> ()
+    end ;
+
       self#log#f 3 "Connecting mount %s for %s@%s..." mount user host ;
 
       set_host conn host ;
