@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -51,39 +51,37 @@ end
 (** Output in an MP3 file *)
 
 class to_file
+  ~append ~perm ~dir_perm
+  ~reload_delay ~reload_predicate ~reload_on_metadata
   ~filename ~samplerate ~bitrate ~quality ~stereo ~autostart source =
 object (self)
   inherit
     [Lame.encoder] Output.encoded
          ~name:filename ~kind:"output.file" ~autostart source
+  inherit File_output.to_file
+            ~reload_delay ~reload_predicate ~reload_on_metadata
+            ~append ~perm ~dir_perm filename as to_file
   inherit base
 
-  method reset_encoder encoder m = ""
-
-  val mutable fd = None
+  method reset_encoder _ m =
+    to_file#on_reset_encoder ;
+    to_file#set_metadata (Hashtbl.find (Hashtbl.copy m)) ;
+    ""
 
   method output_start =
-    assert (fd = None) ;
     let enc = create_encoder ~quality ~bitrate ~stereo ~samplerate in
-      fd <- Some (open_out filename) ;
-      encoder <- Some enc
+      encoder <- Some enc ;
+      to_file#file_output_start
 
   method output_stop =
-    match fd with
-      | None -> assert false
-      | Some v -> close_out v ; fd <- None
-
-  method send b =
-    match fd with
-      | None -> assert false
-      | Some fd -> output_string fd b
+    to_file#file_output_stop
 
   method output_reset = ()
 end
 
 let () =
   Lang.add_operator "output.file.mp3"
-    [ "start",
+    ([ "start",
       Lang.bool_t, Some (Lang.bool true),
       Some "Start output threads on operator initialization." ;
 
@@ -105,14 +103,9 @@ let () =
       "stereo",
       Lang.bool_t,
       Some (Lang.bool true),
-      None;
+      None]
 
-      "",
-      Lang.string_t,
-      None,
-      Some "Filename where to output the MP3 stream." ;
-
-      "", Lang.source_t, None, None ]
+      @ File_output.proto @ ["", Lang.source_t, None, None ])
     ~category:Lang.Output
     ~descr:"Output the source's stream as an MP3 file."
     (fun p ->
@@ -124,5 +117,15 @@ let () =
        let bitrate = e Lang.to_int "bitrate" in
        let filename = Lang.to_string (Lang.assoc "" 1 p) in
        let source = Lang.assoc "" 2 p in
+       let append = Lang.to_bool (List.assoc "append" p) in
+       let perm = Lang.to_int (List.assoc "perm" p) in
+       let dir_perm = Lang.to_int (List.assoc "dir_perm" p) in
+       let reload_predicate = List.assoc "reopen_when" p in
+       let reload_delay = Lang.to_float (List.assoc "reopen_delay" p) in
+       let reload_on_metadata =
+         Lang.to_bool (List.assoc "reopen_on_metadata" p)
+       in
          ((new to_file ~filename
+             ~append ~perm ~dir_perm
+             ~reload_delay ~reload_predicate ~reload_on_metadata
              ~quality ~bitrate ~samplerate ~stereo ~autostart source):>source))
