@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -62,8 +62,9 @@ object (self)
     if ns = [] then
       ns <- Server.register [self#id] kind ;
     self#set_id (Server.to_string ns) ;
-    Server.add ~ns "skip" (fun _ -> self#skip ; "Done") ;
-    Server.add ~ns "metadatas"
+    Server.add ~ns "skip" (fun _ -> self#skip ; "Done") 
+               ~descr:"Skip current song.";
+    Server.add ~ns "metadata" ~descr:"Print current metadata."
       (fun _ ->
          let q = self#metadata_queue in
            (fst (Queue.fold
@@ -74,15 +75,18 @@ object (self)
                               (Request.string_of_metadata m) in
                         s,(i-1))
                    ("",(Queue.length q)) q))) ;
-    Server.add ~ns "remaining"
+    Server.add ~ns "remaining" ~descr:"Display estimated remaining time."
       (fun _ ->
          let r = source#remaining in
            if r < 0 then "(undef)" else
              let t = Fmt.seconds_of_ticks r in
                Printf.sprintf "%.2f" t) ;
-    Server.add ~ns "start" (fun _ -> start_output <- true ; "") ;
-    Server.add ~ns "stop" (fun _ -> stop_output <- true ; "") ;
-    Server.add ~ns "status" (fun _ -> if does_output then "on" else "off") ;
+    Server.add ~ns "start" (fun _ -> start_output <- true ; "") 
+               ~descr:"Start output." ;
+    Server.add ~ns "stop" (fun _ -> stop_output <- true ; "") 
+               ~descr:"Stop output."  ;
+    Server.add ~ns "status" (fun _ -> if does_output then "on" else "off") 
+               ~descr:"Get status.";
 
     (* Get our source ready *)
     source#get_ready ((self:>operator)::activation) ;
