@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -45,12 +45,39 @@ object (self)
     else
       Float_pcm.create_buffer 1 (Fmt.samples_per_frame())
 
-  val mutable os = Ogg.Stream.create ()
+  val mutable _os = None
+  val mutable flush = false
 
-  method virtual new_encoder : Vorbis.Encoder.t
+  method new_os ?(tags=None) enc = 
+    let os = Ogg.Stream.create () in
+    let tags = 
+      match tags with
+        | None -> Vorbis.tags ~artist:"The Savonet Team" 
+                              ~title:"Liquidsoap Stream" ()
+        | Some t -> t
+    in
+    Vorbis.Encoder.headerout enc os tags ;
+    (* Must flush headers first.. *)
+    flush <- true ;
+    _os <- Some os
 
-  (** Do we have to flush before paging out the next data? *)
-  val mutable must_flush = false
+  method end_of_os encoder = 
+    match _os with
+      | None -> ""
+      | Some os ->
+         let encoder = Utils.get_some encoder in
+         Vorbis.Encoder.end_of_stream encoder os ;
+         let f = Ogg.Stream.flush os in
+         _os <- None ;
+         f
+
+  method get_os e = 
+    match _os with
+      | Some s -> s
+      | None -> self#new_os e ;
+                Utils.get_some _os
+
+  method virtual new_encoder : bool -> Vorbis.Encoder.t
 
   method reset_encoder encoder m =
     let get h k =
@@ -72,29 +99,29 @@ object (self)
                | Not_found -> title)
         | None -> "Unknown"
     in
-      Vorbis.Encoder.end_of_stream encoder os;
-      let flushed = Ogg.Stream.flush os in
-        os <- Ogg.Stream.create ();
-        (* Vorbis.Encoder.reset encoder; *)
-        Vorbis.Encoder.headerout self#new_encoder os
-          (Vorbis.tags
-             ?title:(getd m "title" def_title)
-             ?artist:(get m "artist")
-             ?genre:(get m "genre")
-             ?date:(get m "date")
-             ?album:(get m "album")
-             ?tracknumber:(get m "tracknum")
-             ?comment:(get m "comment")
-             ());
-        (* The header must be on its own page. *)
-        must_flush <- true;
+        let flushed = self#end_of_os (Some encoder) in
+        let encoder = self#new_encoder stereo in
+        self#new_os 
+          ~tags:(Some (Vorbis.tags
+                         ?title:(getd m "title" def_title)
+                         ?artist:(get m "artist")
+                         ?genre:(get m "genre")
+                         ?date:(get m "date")
+                         ?album:(get m "album")
+                         ?tracknumber:(get m "tracknum")
+                         ?comment:(get m "comment")
+                         ())) encoder  ;
         flushed
 
   method encode e b start len =
     let b =
       if stereo then b else begin
         for i = start to start+len-1 do
-          tmp.(0).(i) <- (b.(0).(i) +. b.(1).(i)) /. 2.
+	  let n = Fmt.channels () in
+	  let f i = 
+	    Array.fold_left (fun x y -> x +. y.(i)) 0. b
+	  in
+          tmp.(0).(i) <- f i /. (float_of_int n)
         done ;
         tmp
       end
@@ -104,17 +131,15 @@ object (self)
         (float freq /. samples_per_second)
         b start len
     in
-    let f =
-      if must_flush then
-        (
-          must_flush <- false;
-          Ogg.Stream.flush os
-        )
-      else
-        ""
-    in
+    let os = self#get_os e in
+      let f = 
+        if flush then
+	  Ogg.Stream.flush os
+	else
+	  ""
+      in
       Vorbis.Encoder.encode_buffer_float e os buf 0 (Array.length buf.(0));
-      f ^ Ogg.Stream.pagesout os
+      f ^ (Ogg.Stream.pagesout os)
 end
 
 (** Output in an Ogg/vorbis file. *)
@@ -131,14 +156,16 @@ object (self)
             ~append ~perm ~dir_perm filename as to_file
   inherit base freq stereo as base
 
-  method new_encoder =
+  method new_encoder stereo =
+    let channels = 
+      if not stereo then 1 else Fmt.channels () in
     let enc =
       (* TODO: log message when the creation of the encoder fails *)
       let nom,min,max = bitrate in
       match mode with
         | ABR
-        | CBR -> Vorbis.Encoder.create (Fmt.channels ()) freq max nom min
-        | VBR -> Vorbis.Encoder.create_vbr (Fmt.channels ()) freq quality
+        | CBR -> Vorbis.Encoder.create channels freq max nom min
+        | VBR -> Vorbis.Encoder.create_vbr channels freq quality
     in
       encoder <- Some enc ;
       enc
@@ -146,23 +173,16 @@ object (self)
   method reset_encoder enc m =
     to_file#on_reset_encoder ;
     to_file#set_metadata (Hashtbl.find (Hashtbl.copy m)) ;
-    (* Avoid multiple resets *)
-    if must_flush then "" else base#reset_encoder enc m
+    base#reset_encoder enc m
 
-  method output_start =
-    to_file#file_output_start ;
-    let enc = self#new_encoder in
-    let tags =
-      Vorbis.tags ~artist:"The Savonet Team" ~title:"Liquidsoap Stream" ()
-    in
-      Vorbis.Encoder.headerout enc os tags ;
-      must_flush <- true
+  method output_start = 
+    ignore(self#new_encoder stereo) ;
+    to_file#file_output_start 
 
   method output_stop =
-    Vorbis.Encoder.end_of_stream (Utils.get_some encoder) os ;
-    to_file#send (Ogg.Stream.flush os) ;
-    os <- Ogg.Stream.create () ;
-    to_file#file_output_stop
+    let f = base#end_of_os encoder in
+    to_file#send f ;
+    to_file#file_output_stop 
 
 end
 
@@ -218,7 +238,7 @@ let () =
 
 let () =
   Lang.add_operator "output.file.vorbis.cbr" (* Constant BitRate *)
-    (vorbis_proto @ File_output.proto @ [
+    (vorbis_proto @ [
       "start",
       Lang.bool_t, Some (Lang.bool true),
       Some "Start output on operator initialization." ;
@@ -226,9 +246,8 @@ let () =
       "bitrate",
       Lang.int_t,
       Some (Lang.int 128),
-      Some "Bitrate (in kbps).";
-
-      "", Lang.source_t, None, None ])
+      Some "Bitrate (in kbps)."] @ File_output.proto @
+      ["", Lang.source_t, None, None ])
     ~category:Lang.Output
     ~descr:("Output the source stream as an Ogg Vorbis file "
             ^ "in Constant BitRate mode.")
@@ -256,7 +275,7 @@ let () =
 
 let () =
   Lang.add_operator "output.file.vorbis" (* Variable BitRate *)
-    (vorbis_proto @ File_output.proto @ [
+    (vorbis_proto @ [
       "start",
       Lang.bool_t, Some (Lang.bool true),
       Some "Start output on operator initialization." ;
@@ -264,9 +283,9 @@ let () =
       "quality",
       Lang.float_t,
       Some (Lang.float 2.),
-      Some ("Desired quality level, currently from -1. to 10. (low to high).");
+      Some ("Desired quality level, currently from -1. to 10. (low to high).")] 
 
-      "", Lang.source_t, None, None ])
+      @ File_output.proto @ ["", Lang.source_t, None, None ])
     ~category:Lang.Output
     ~descr:("Output the source stream as an Ogg Vorbis file "
             ^ "in Variable BitRate mode.")
