@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ let annotate s ~log maxtime =
       [Request.indicator ~metadata:(Utils.hashtbl_of_list metadata) uri]
   with
     | Error
-    | Stream.Failure -> log "annotate: syntax error" ; []
+    | Stream.Failure | Stream.Error _ -> log "annotate: syntax error" ; []
 
 let () =
   Request.protocols#register "annotate"
