@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 open Source
 open Dtools
 
-let dlog = Log.make ["protocols";"extproto"]
+let dlog = Log.make ["protocols";"external"]
 
 let resolve proto program command s ~log maxtime =
   let s = proto ^ ":" ^ s in
