@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -189,7 +189,11 @@ type indicator = {
 
 type status = Idle | Resolving | Ready | Playing | Destroyed
 
-type t = {
+type audio = [ `Audio ]
+type raw   = [ `Raw ]
+type flags = [ audio | raw ]
+
+type _t = {
   id : int ;
   initial_uri : string ;
   audio : bool ;
@@ -212,6 +216,10 @@ type t = {
   mutable indicators : indicator list list ;
   mutable decoder : (unit -> Decoder.decoder) option ;
 }
+type +'a t = _t constraint 'a = [<flags]
+
+let to_raw   x = x
+let to_audio x = assert x.audio ; x
 
 let indicator ?(metadata=Hashtbl.create 10) ?temporary s = {
   string = home_unrelate s ;
@@ -309,11 +317,11 @@ let local_check t =
       let indicator = peek_indicator t in
       let name = indicator.string in
         match Decoder.search_valid name with
-          | Some f ->
+          | Some (format,f) ->
               t.decoder <- Some f ;
               mresolvers#iter
-                (fun format resolver ->
-                   let ans = resolver name in
+                (fun _ resolver ->
+                   let ans = resolver ~format name in
                      List.iter
                        (fun (k,v) ->
                           (* XXX Policy ? Never/always overwrite ? *)
@@ -353,6 +361,29 @@ let get_filename t =
   else
     None
 
+(** Length *)
+let dresolvers_doc =
+  "Methods to extract duration from a file."
+let dresolvers =
+  Plug.create ~doc:dresolvers_doc ~insensitive:true
+    "audio file formats (duration)"
+
+exception Duration of float
+let duration file =
+  try
+    dresolvers#iter
+      (fun format resolver ->
+        try
+         let ans = resolver file in
+         raise (Duration ans)
+        with 
+	  | Duration e -> raise (Duration e) 
+	  | _ -> () ) ; 
+      raise Not_found
+  with
+    | Duration d -> d
+
+
 let update_metadata t =
   let replace = Hashtbl.replace t.root_metadata in
     replace "rid" (string_of_int t.id) ;
@@ -462,7 +493,7 @@ let resolving_requests () =
 
 (** Creation *)
 
-let create ?(metadata=[]) ?(audio=true) ?(persistent=false) ?(indicators=[]) u =
+let _create ~audio ?(metadata=[]) ?(persistent=false) ?(indicators=[]) u =
   Mutex.lock lock ;
   let rid = next () in
     if rid = -1 then
@@ -494,6 +525,9 @@ let create ?(metadata=[]) ?(audio=true) ?(persistent=false) ?(indicators=[]) u =
           (if indicators=[] then [indicator u] else indicators) ;
         Some t
 
+let create = _create ~audio:true
+let create_raw = _create ~audio:false
+
 let on_air t =
   t.on_air <- Some (Unix.time ()) ;
   t.status <- Playing ;
@@ -593,7 +627,7 @@ let resolve t timeout =
   in
   let excess = (Unix.time ()) -. maxtime in
     if excess > 0. then
-      log#f 2 "Time limit exceeded by %f secs!" excess ;
+      log#f 2 "Time limit exceeded by %.2f secs!" excess ;
     t.resolving <- None ;
     if result <> Resolved then t.status <- Idle else t.status <- Ready ;
     result
