@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,19 +26,32 @@
   * The request is actually the file abstraction in liquidsoap, and you
   * should think about using it as much as possible. *)
 
-type t
+type audio = [ `Audio ]
+type raw   = [ `Raw ]
+type flags = [ audio | raw ]
+
+type 'a t constraint 'a = [<flags]
 type metadata = (string,string) Hashtbl.t
 
 type indicator
 val indicator : ?metadata:metadata -> ?temporary:bool -> string -> indicator
 
-(** [audio] means that we expect an audio file. In case [audio] is set,
+(** For audio requests,
   * resolving includes testing that the file can actually be decoded. *)
 val create :
   ?metadata:((string*string) list) -> 
-  ?audio:bool -> ?persistent:bool ->
+  ?persistent:bool ->
   ?indicators:(indicator list) -> string ->
-  t option
+  (audio t) option
+val create_raw :
+  ?metadata:((string*string) list) -> 
+  ?persistent:bool ->
+  ?indicators:(indicator list) -> string ->
+  (raw t) option
+
+(** This should eventually disappear, but is useful in Lang for now. *)
+val to_raw : 'a t -> raw t
+val to_audio : 'a t -> audio t
 
 (** Destroying of a requests causes its file to be deleted if it's a temporary
   * one, for example a downloaded file. If the metadata ["persistent"] is
@@ -46,10 +59,10 @@ val create :
   * Persistent sources are useful for static URIs (see below for the definition
   * of staticity,
   * and in [src/sources/one_file.ml] for an example of use). *)
-val destroy : ?force:bool -> t -> unit
+val destroy : ?force:bool -> 'a t -> unit
 
 (** Check if a request is currently being streamed. *)
-val on_air : t -> unit
+val on_air : audio t -> unit
 
 (** {1 General management} *)
 
@@ -60,8 +73,8 @@ val clean : unit -> unit
 
 (** Every request has an id, and you can find a request from its id. *)
 
-val get_id : t -> int
-val from_id : int -> t option
+val get_id : 'a t -> int
+val from_id : int -> audio t option
 
 (** Get the list of requests that are currently
   * alive/on air/being resolved. *)
@@ -94,7 +107,7 @@ val is_static : string -> bool option
   * the Decoder, simply because we want it to occur immediately
   * after resolving. *)
 
-val mresolvers : (string -> (string*string) list) Plug.plug
+val mresolvers : (format:string -> string -> (string*string) list) Plug.plug
 val protocols : protocol Plug.plug
 
 (** Resolving can fail because an URI is invalid, or doesn't refer to a valid
@@ -104,45 +117,55 @@ type resolve_flag = Resolved | Failed | Timeout
 (** [resolve request timeout] tries to resolve the request within [timeout]
   * seconds. If resolving succeeds, [is_ready request] is true and you
   * can get a filename. *)
-val resolve : t -> float -> resolve_flag
+val resolve : 'a t -> float -> resolve_flag
 
 (** [is_ready r] if there's an available local filename. It can be true even if
   * the resolving hasn't been run, if the initial URI was already a local
   * filename. *)
-val is_ready : t -> bool
+val is_ready : 'a t -> bool
 
 (** Return a valid local filename if there is one, which means that the request
   * is ready. *)
-val get_filename : t -> string option
+val get_filename : 'a t -> string option
 
 (** {1 URI manipulation} For protocol plugins. *)
 
-val peek_indicator : t -> string
+val peek_indicator : 'a t -> string
 (** Removes the top URI, possibly destroys the associated file. *)
-val pop_indicator : t -> unit
+val pop_indicator : 'a t -> unit
 
 (** In most of the case you don't peek or pop any URI, you just push the
   * new URIs you computed from [first_uri]. *)
-val push_indicators : t -> indicator list -> unit
+val push_indicators : 'a t -> indicator list -> unit
 
 (** {1 Metadatas} *)
 
 val string_of_metadata : metadata -> string
 val short_string_of_metadata : metadata -> string
-val set_metadata : t -> string -> string -> unit
-val get_metadata : t -> string -> string option
-val set_root_metadata : t -> string -> string -> unit
-val get_root_metadata : t -> string -> string option
-val get_all_metadata : t -> metadata
+val set_metadata : 'a t -> string -> string -> unit
+val get_metadata : 'a t -> string -> string option
+val set_root_metadata : 'a t -> string -> string -> unit
+val get_root_metadata : 'a t -> string -> string option
+val get_all_metadata : 'a t -> metadata
+
+(** {1 Duration} *)
+
+val dresolvers : (string -> float) Plug.plug
+
+(** [duration filename] computes the duration of audio data contained in
+  * [filename]. The computation may be expensive.
+  * @raise Not_found if no duration computation method is found. *)
+val duration : string -> float
+
 
 (** {1 Logging}
   * Every request has a separate log in which its history can be written. *)
 
 type log = (Unix.tm*string) Queue.t
 val string_of_log : log -> string
-val add_log : t -> string -> unit
-val get_log : t -> log
+val add_log : 'a t -> string -> unit
+val get_log : 'a t -> log
 
 (** Return a decoder if the file has been resolved, guaranteed to have
   * available data to deliver. *)
-val get_decoder : t -> Decoder.decoder option
+val get_decoder : audio t -> Decoder.decoder option
