@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -75,6 +75,8 @@ object (self)
     id <- "src_" ^ (string_of_int (Oo.id self))
   method id = id
   method set_id ?(definitive=true) s =
+    (* ID musn't contain "." *)
+    let s = Pcre.substitute ~pat:"\\." ~subst:(fun _ -> "(dot)") s in
     if not definitive_id then ( id <- s ; definitive_id <- definitive ) ;
     (* Sometimes the ID is changed during initialization,
      * in order to make it equal to the server name,
@@ -185,7 +187,7 @@ object (self)
    * Depending whether caching is enabled or not, it calls [#get_frame] directly
    * or tries to get data from the cache frame, filling it if needed.
    * Any source calling [other_source#get should] thus take care of clearing
-   * the cache of the other source ([#clear_cache]) at the end of the output
+   * the cache of the other source ([#advance]) at the end of the output
    * round ([#after_output]). *)
   method get buf =
     assert (Frame.is_partial buf) ;
@@ -227,11 +229,11 @@ object (self)
    * can freeze its state in an unwanted way. *)
   method after_output =
     List.iter (fun s -> s#after_output) sources ;
-    self#clear_cache
+    self#advance
 
   (* Reset the cache frame *)
-  method clear_cache =
-    Frame.clear memo
+  method advance =
+    Frame.advance memo
 
   (** Utils. *)
 
