@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -94,16 +94,16 @@ object
     * In that case, after_output allows the node to wait for its
     * output threads. *)
   method after_output : unit
-  method clear_cache : unit
+  method advance : unit
 
   (** {1 Utilities} *)
 
   (** Create a request with a "source" metadata. *)
   method private create_request :
     ?metadata:((string*string) list) ->
-    ?audio:bool -> ?persistent:bool ->
+    ?persistent:bool ->
     ?indicators:(Request.indicator list) -> string ->
-    Request.t option
+    Request.audio Request.t option
 
   method private log : Dtools.Log.t
 
