@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -34,12 +34,12 @@ exception Error of string
 let log = Dtools.Log.make ["input";"alsa"]
 
 class mic device =
-   let buffer_length = Fmt.samples_per_frame () in
-   let buffer_chans = Fmt.channels () in
-   (* Only one block is OK, but 10 doesn't cost much and could allow
-    * some timing errors to be absorbed.. I can't hear any difference anyway. *)
-   let nb_blocks = 10 in
-   let blank = Array.init buffer_chans (fun _ -> Array.make buffer_length 0.) in
+  let buffer_length = Fmt.samples_per_frame () in
+  let buffer_chans = Fmt.channels () in
+  (* Only one block is OK, but 10 doesn't cost much and could allow
+   * some timing errors to be absorbed.. I can't hear any difference anyway. *)
+  let nb_blocks = 10 in
+  let blank = Array.init buffer_chans (fun _ -> Array.make buffer_length 0.) in
 object (self)
   inherit active_source
 
@@ -54,7 +54,8 @@ object (self)
   val buffer = Array.make nb_blocks [||]
   initializer
     for i = 0 to nb_blocks - 1 do
-      buffer.(i) <- Array.init buffer_chans (fun _ -> Array.make buffer_length 0.)
+      buffer.(i) <-
+        Array.init buffer_chans (fun _ -> Array.make buffer_length 0.)
     done
   val mutable read = 0
   val mutable write = 0
@@ -72,34 +73,28 @@ object (self)
   method output_get_ready =
     sleep <- false ;
     read <- 0 ; write <- 0 ;
-    let reader = Mutex.create () in
-      Mutex.lock reader ;
-      ignore (Tutils.create (fun l -> self#writer l) reader "alsa_capture") ;
-      (* Wait for the first buffer input. *)
-      Mutex.lock reader
+    ignore (Tutils.create (fun () -> self#writer) () "alsa_capture")
 
-  method writer reader_lock =
+  method writer =
     self#log#f 3 "Using ALSA %s." (Alsa.get_version ()) ;
     let dev = Pcm.open_pcm device [Pcm.Capture] [] in
     let params = Pcm.get_params dev in
-      (
-        try
-          Pcm.set_access dev params Pcm.Access_rw_noninterleaved ;
-          Pcm.set_format dev params Pcm.Format_float
-        with
-          | _ ->
-              (* If we can't get floats we fallback on interleaved s16le *)
-              self#log#f 2 "Falling back on interleaved S16LE";
-              Pcm.set_access dev params Pcm.Access_rw_interleaved ;
-              Pcm.set_format dev params Pcm.Format_s16_le ;
-              read_fun <-
+      begin try
+        Pcm.set_access dev params Pcm.Access_rw_noninterleaved ;
+        Pcm.set_format dev params Pcm.Format_float
+      with
+        | _ ->
+            (* If we can't get floats we fallback on interleaved s16le *)
+            self#log#f 2 "Falling back on interleaved S16LE";
+            Pcm.set_access dev params Pcm.Access_rw_interleaved ;
+            Pcm.set_format dev params Pcm.Format_s16_le ;
+            read_fun <-
               (fun pcm buf ofs len ->
                  let sbuf = String.create (2 * 2 * len) in
                  let r = Pcm.readi pcm sbuf 0 len in
                    Float_pcm.from_s16le buf ofs sbuf 0 r;
-                   r
-              )
-      );
+                   r)
+      end ;
       sample_freq <-
         Pcm.set_rate_near dev params sample_freq Dir_eq; (* TODO: resample *)
       Pcm.set_channels dev params buffer_chans ;
@@ -124,15 +119,17 @@ object (self)
         (* Fill the first block *)
         fill 0 ;
         write <- 1 ;
-        Mutex.unlock reader_lock ;
         (* Filling loop *)
         while not sleep do
-          if read <> write &&
-             write mod nb_blocks = read mod nb_blocks then begin
-               (* Wait for the reader to read the block we fancy *)
-               Thread.delay (Fmt.seconds_per_frame () /. 2.)
-          end else
-            ( fill write ; write <- (write + 1) mod (2*nb_blocks) )
+          if
+            read <> write &&
+            write mod nb_blocks = read mod nb_blocks
+          then begin
+            (* Wait for the reader to read the block we fancy *)
+            Thread.delay (Fmt.seconds_per_frame () /. 2.)
+          end else begin
+            fill write ; write <- (write + 1) mod (2*nb_blocks)
+          end
         done
 
   method get_frame buf =
@@ -160,4 +157,4 @@ object (self)
 
 end
 
-(* It is registered in Alsa_io. *)
+(* This source is registered in Alsa_io. *)
