@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -57,11 +57,12 @@ let stream_decoders : (sink -> unit) Plug.plug =
 
 (** Utilities for reading icy metadata *)
 
+let log = Dtools.Log.make ["readmeta"]
 let read_metadata () = let old_chunk = ref "" in fun socket ->
   let size =
     let buf = " " in
     let s = Unix.read socket buf 0 1 in
-      assert (s=1) ;
+      assert (s=1) ; (* NON *)
       int_of_char buf.[0]
   in
   let size = 16*size in
@@ -70,7 +71,7 @@ let read_metadata () = let old_chunk = ref "" in fun socket ->
     let rec read pos =
       if pos=size then buf else
         let p = Unix.read socket buf pos (size-pos) in
-          assert (p>0) ;
+          assert (p>0) ; (* NON *)
           read (pos+p)
     in
       read 0
@@ -92,7 +93,13 @@ let read_metadata () = let old_chunk = ref "" in fun socket ->
         parse (String.sub s (close+1) ((String.length s)-close-1))
     with _ -> ()
   in
-    if chunk = "" || chunk = !old_chunk then None else begin
+    if chunk = "" then begin
+      (* log#f 4 "Empty chunk!" ; *)
+      None
+    end else if chunk = !old_chunk then begin
+      (* log#f 4 "Redundant chunk (%S)!" chunk ; *)
+      None
+    end else begin
       old_chunk := chunk ;
       parse chunk ;
       Some h
@@ -101,10 +108,10 @@ let read_metadata () = let old_chunk = ref "" in fun socket ->
 let read_line socket =
   let ans = ref "" in
   let c = String.create 1 in
-    assert (Unix.read socket c 0 1 = 1);
+    assert (Unix.read socket c 0 1 = 1); (* NON *)
     while c <> "\n" do
       ans := !ans ^ c;
-      assert (Unix.read socket c 0 1 = 1);
+      assert (Unix.read socket c 0 1 = 1); (* NON *)
     done;
     String.sub !ans 0 (String.length !ans - 1)
 
@@ -192,7 +199,8 @@ module Generated = Generated.From_Float_pcm_Generator
 (* Used to handle redirections. *)
 exception Redirection of string
 
-class http ~playlist_mode ~autostart ~bufferize ~timeout ~max url =
+class http ~playlist_mode ~poll_delay ~timeout
+           ~bind_address ~autostart ~bufferize ~max url =
   let abg_max_len = Fmt.samples_of_seconds max in
 object (self)
   inherit Source.source
@@ -202,19 +210,26 @@ object (self)
 
   method stype = Source.Fallible
 
-  val polling = Mutex.create ()
-  val mutable sleeping = true
+  (** The poll_should_stop says that the current polling thread should stop,
+    * because #sleep was called. The subsequent #wake_up call will wake for the
+    * thread to set poll_should_stop to false again and exit.
+    * The condition/lock devices are only useful for avoiding an active
+    * wait in #wake_up, because the race conditions are completely harmless. *)
+  val mutable poll_should_stop = false
+  val polling_lock = Mutex.create ()
+  val polling_cond = Condition.create ()
+
   val mutable connected = false
   val mutable relaying = autostart
   val mutable playlist_mode = playlist_mode
 
+
   (* Insert metadata *)
   method insert_metadata m =
     self#log#f 3 "New metadata chunk \"%s -- %s\""
                 (try Hashtbl.find m "artist" with _ -> "?")
                 (try Hashtbl.find m "title" with _ -> "?") ;
-    metadata <- m ;
-    metadata_pos <- Fmt.ticks_of_samples (Generator.length abg)
+    metadata <- (Fmt.ticks_of_samples (Generator.length abg), m) :: metadata ;
 
   (* Feed the buffer generator *)
 
@@ -228,11 +243,17 @@ object (self)
      * Doing 2 instead of 1 leads to ugly sound.
      * The problem is:
      * we don't know when the source will really be asked for data... *)
-    while Generator.length abg >= abg_max_len do
-      Mutex.unlock lock ;
-      Thread.delay (max /. 3.) ;
-      Mutex.lock lock
-    done ;
+    if Generator.length abg >= abg_max_len then
+      begin
+        Mutex.unlock lock ;
+        Thread.delay (max /. 3.) ;
+        Mutex.lock lock ;
+        if Generator.length abg >= abg_max_len then
+        (* Here, we drop some data after the maximun buffer has been filled.
+         * Delaying the function can lead to deconnection/connection cycles
+         * when the source is not pulled. *)
+          Generator.remove abg (Generator.length abg - abg_max_len)
+      end ;
     Generator.feed abg ~sample_freq data ;
     Mutex.unlock lock
 
@@ -262,14 +283,16 @@ object (self)
         | "" -> ""
         | _ -> "Authorization: Basic " ^ (Utils.encode64 auth) ^ "\r\n"
     in
-    let request = 
+    let request =
       Printf.sprintf
         "%sUser-Agent: liquidsoap/%s (%s; ocaml %s)\r\n%sIcy-MetaData:1\r\n\r\n"
         req Configure.version Sys.os_type Sys.ocaml_version auth
     in
       self#log#f 4 "Connecting to <http://%s:%d%s>..." host port mount ;
       try
-        let socket = Http.connect ~timeout:(Some timeout) host port in
+        let socket =
+          Http.connect ~bind_address ~timeout:(Some timeout) host port
+        in
           try
             let (_, status, status_msg), fields = Http.request socket request in
             let content_type =
@@ -293,34 +316,34 @@ object (self)
                 List.assoc "transfer-encoding" fields = "chunked"
               with _ -> false
             in
-              if status = 301 || status = 302 || status = 303 || status = 307 then
-                (
-                  let location =
-                    try
-                      List.assoc "location" fields
-                    with
-                      | Not_found -> raise Internal
-                  in
-                    self#log#f 4 "Redirected to %s" location;
-                    raise (Redirection location)
-                );
-              if status <> 200 then
-                (
-                  self#log#f 4 "Could not get file: %s" status_msg;
-                  raise Internal
-                );
-              let play_track (m,uri) = 
+              if
+                status = 301 || status = 302 || status = 303 || status = 307
+              then begin
+                let location =
+                  try
+                    List.assoc "location" fields
+                  with
+                    | Not_found -> raise Internal
+                in
+                  self#log#f 4 "Redirected to %s" location;
+                  raise (Redirection location)
+              end ;
+              if status <> 200 then begin
+                self#log#f 4 "Could not get file: %s" status_msg;
+                raise Internal
+              end ;
+              let play_track (m,uri) =
                 let metas = Hashtbl.create 2 in
                   List.iter (fun (a,b) -> Hashtbl.add metas a b) m;
                   self#insert_metadata metas;
                   self#private_connect uri
               in
-              let randomize playlist = 
+              let randomize playlist =
                 let aplay = Array.of_list playlist in
                   Utils.randomize aplay;
-                  Array.to_list aplay 
+                  Array.to_list aplay
               in
-              let playlist_process playlist = 
+              let playlist_process playlist =
                 try
                   match playlist_mode with
                     | Random ->  play_track (List.hd (randomize playlist))
@@ -330,29 +353,30 @@ object (self)
                 with
                   | Failure hd -> raise Not_found
               in
-              let test_playlist parser = 
+              let test_playlist parser =
                 let content = Http.read socket None in
                 let playlist = parser content in
-                  match playlist with 
+                  match playlist with
                     | [] -> raise Not_found
                     | _ -> () ;
                            playlist_process playlist
               in
                 try
-                  self#log#f 4 "Trying playlist parser for mime %s" content_type;
+                  self#log#f 4
+                    "Trying playlist parser for mime %s" content_type ;
                   match Playlist_parser.parsers#get content_type with
                     | None -> raise Not_found
                     | Some plugin ->
                         test_playlist plugin.Playlist_parser.parser
                 with
                   | Not_found ->
-                      (* Trying playlist auto parsing in case 
+                      (* Trying playlist auto parsing in case
                        * of content type text/plain *)
                       if content_type = "text/plain" then
                         begin
                           try
-                            let f (_,b) = b in 
-                              test_playlist (fun x -> f (Playlist_parser.search_valid x))
+                            test_playlist
+                              (fun x -> snd (Playlist_parser.search_valid x))
                           with
                             | Not_found -> ()
                         end;
@@ -383,49 +407,70 @@ object (self)
   method poll =
     (* Try to read the stream *)
     if relaying then self#connect url ;
-    if sleeping then
-      Mutex.unlock polling
-    else
-      ( Thread.delay 1. ; self#poll )
+    if poll_should_stop then begin
+      poll_should_stop <- false ;
+      Condition.signal polling_cond
+    end else begin
+      (* TODO Use Duppy instead of a separate thread ? *)
+      Thread.delay poll_delay ;
+      self#poll
+    end
 
   val mutable ns = []
 
   method wake_up _ =
-    (* Wait for the polling thread to return *)
-    Mutex.lock polling ;
-    sleeping <- false ;
+    (* Wait for the old polling thread to return, then create a new one. *)
+    Tutils.wait polling_cond polling_lock (fun () -> not poll_should_stop) ;
     ignore (Tutils.create (fun () -> self#poll) () "http polling") ;
     if ns = [] then
       ns <- Server.register [self#id] "input.http" ;
     self#set_id (Server.to_string ns) ;
-    Server.add ~ns "start" ~usage:"start" (fun _ -> relaying <- true ; "Done") ;
-    Server.add ~ns "stop" ~usage:"stop" (fun _ -> relaying <- false ; "Done")
+    Server.add ~ns "start" ~usage:"start" ~descr:"Start the source, if needed." 
+       (fun _ -> relaying <- true ; "Done") ;
+    Server.add ~ns "stop" ~usage:"stop" ~descr:"Stop the source if streaming."
+       (fun _ -> relaying <- false ; "Done")
 
-  method sleep =
-    sleeping <- true
+  method sleep = poll_should_stop <- true
 
 end
 
 let () =
     Lang.add_operator "input.http"
       ~category:Lang.Input
-      ~descr:("Forwards the given http stream. The relay can be "^
-              "paused/resumed using the start/stop telnet commands.")
+      ~descr:"Forwards the given http stream. The relay can be \
+              paused/resumed using the start/stop telnet commands."
       [ "autostart", Lang.bool_t, Some (Lang.bool true),
         Some "Initially start relaying or not." ;
+
+        "bind_address", Lang.string_t, Some (Lang.string ""),
+        Some "Address to bind on the local machine. \
+              This option can be useful if \
+              your machine is bound to multiple IPs. \
+              Empty means no bind address." ;
+
         "buffer", Lang.float_t, Some (Lang.float 2.),
-	Some "Duration of the pre-buffered data." ;
-	"timeout", Lang.float_t, Some (Lang.float 2.),
-	Some "Timeout for http connection." ;
-	"playlist_mode", Lang.string_t, (Some (Lang.string "normal")),
-	            (Some "normal|random|randomize|first") ;
+        Some "Duration of the pre-buffered data." ;
+
+        "timeout", Lang.float_t, Some (Lang.float 10.),
+        Some "Timeout for http connection." ;
+
+        "playlist_mode", Lang.string_t, Some (Lang.string "normal"),
+        Some "Valid modes are \"normal\", \"random\", \"randomize\" \
+              and \"first\". The first ones have the same meaning as for \
+              the mode parameter of the playlist operator. The last one \
+              discards all entries but the first one." ;
+
+        "poll_delay", Lang.float_t, Some (Lang.float 2.),
+        Some "Polling delay." ;
+
         "max", Lang.float_t, Some (Lang.float 10.),
         Some "Maximum duration of the buffered data." ;
+
         "", Lang.string_t, None,
         Some "URL of an http stream (default port is 80)." ]
       (fun p ->
-           let playlist_mode = 
-	     let s = List.assoc "playlist_mode" p in
+           let playlist_mode =
+             let s = List.assoc "playlist_mode" p in
            match Lang.to_string s with
              | "random" -> Random
              | "first" -> First
@@ -436,11 +481,19 @@ let () =
             (Lang.Invalid_value
                (s,
                 "valid values are 'random', 'randomize', 'normal' and 'first'"))
-         in 
+         in
          let url = Lang.to_string (List.assoc "" p) in
          let autostart = Lang.to_bool (List.assoc "autostart" p) in
+         let bind_address = Lang.to_string (List.assoc "bind_address" p) in
+         let bind_address =
+           match bind_address with
+             | "" -> None
+             | s -> Some s
+         in
          let bufferize = Lang.to_float (List.assoc "buffer" p) in
-	 let timeout = Lang.to_float (List.assoc "timeout" p) in
+         let timeout = Lang.to_float (List.assoc "timeout" p) in
          let max = Lang.to_float (List.assoc "max" p) in
-           ((new http ~playlist_mode ~autostart ~bufferize ~timeout ~max url)
+         let poll_delay = Lang.to_float (List.assoc "poll_delay" p) in
+           ((new http ~playlist_mode ~timeout ~autostart
+                      ~bind_address ~poll_delay ~bufferize ~max url)
               :>Source.source))
