@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -71,27 +71,26 @@ let rec list_files (log : Log.t) dir =
   in
     files@(List.concat (List.map (fun d -> list_files log d) dirs))
 
-class virtual vplaylist reload random playlist_uri timeout =
+class virtual vplaylist mime reload random playlist_uri timeout =
 object (self)
 
   method virtual is_valid : string -> bool
   method virtual stype : Source.source_t
   method virtual id : string
   method virtual set_id : ?definitive:bool -> string -> unit
-  method virtual copy_queue : Request.t list
+  method virtual copy_queue : Request.audio Request.t list
   method virtual create_request :
     ?metadata:((string*string) list) ->
-    ?audio:bool -> ?persistent:bool ->
+    ?persistent:bool ->
     ?indicators:(Request.indicator list) -> string ->
-    Request.t option
+    Request.audio Request.t option
   method virtual log : Dtools.Log.t
 
   (** How to get the playlist. *)
   val mutable playlist_uri = playlist_uri
 
-  (** Current playlist, containing the same files, but possibly shuffled.
-    The complex type is/was for a possible feature with strider. *)
-  val mutable playlist = ref [| |]
+  (** Current playlist, containing the same files, but possibly shuffled. *)
+  val playlist = ref [| |]
 
   (** Index of the current file. *)
   val mutable index_played = -1
@@ -119,7 +118,6 @@ object (self)
   (** (re-)read playlist_file and update datas.
     [reload] should be true except on first load, in that case playlist
     resolution failures won't result in emptying the current playlist.*)
-
   method load_playlist ?(uri=playlist_uri) reload =
     let _playlist =
       let read_playlist filename =
@@ -134,49 +132,89 @@ object (self)
               really_input channel content 0 length;
               (* Close the file now, I don't need it anymore.. *)
               close_in channel ;
-              let (format,playlist) = Playlist_parser.search_valid content in
-                self#log#f 3 "Playlist treated as format %s" format ;
+              let (format,playlist) =
+                match mime with
+                  | "" ->
+                      self#log#f 3
+                        "No mime type specified, trying autodetection." ;
+                      Playlist_parser.search_valid content
+                  | x ->
+                      begin match Playlist_parser.parsers#get x with
+                        | Some plugin ->
+                            (x,plugin.Playlist_parser.parser content)
+                        | None ->
+                            self#log#f 3
+                              "Unknown mime type, trying autodetection." ;
+                            Playlist_parser.search_valid content
+                      end
+              in
+                self#log#f 3 "Playlist treated as format %s" format  ;
                 List.map (fun (_,x) -> x) playlist
           with
-            | _ -> []
+            | e ->
+                self#log#f 3
+                  "Could not parse playlist: %s" (Printexc.to_string e) ;
+                []
       in
         self#log#f 3 "Loading playlist..." ;
-        match Request.create ~audio:false uri with
+        match Request.create_raw uri with
           | None ->
               self#log#f 2 "Could not resolve playlist URI %S!" uri ;
               []
           | Some req ->
-              if Request.Resolved = Request.resolve req timeout then
-                let l =
-                  read_playlist (Utils.get_some (Request.get_filename req))
-                in
-                  Request.destroy req ;
-                  l
-              else begin
-                self#log#f 2 "Timeout when resolving playlist URI %S!" uri ;
-                Request.destroy req ;
-                []
-              end
+              match Request.resolve req timeout with
+                | Request.Resolved ->
+                    let l =
+                      read_playlist (Utils.get_some (Request.get_filename req))
+                    in
+                    Request.destroy req ;
+                    l
+                | e ->
+                    let reason =
+                      match e with
+                        | Request.Timeout -> "Timeout"
+                        | Request.Failed -> "Failed"
+                        | Request.Resolved -> assert false
+                    in
+                    self#log#f 2
+                      "%s when resolving playlist URI %S!"
+                      reason uri ;
+                    Request.destroy req ;
+                    []
     in
-      if _playlist <> [] || not reload then begin
+      (* TODO distinguish error and empty if fallible *)
+      if _playlist = [] && reload then
+        self#log#f 3 "Got an empty list: keeping the old one."
+      else begin
         (* Don't worry if a reload fails,
          * otherwise, the source type must be aware of the failure *)
         Mutex.lock mylock ;
         assert (not (self#stype = Infallible && _playlist = [])) ;
         playlist := rev_array_of_list _playlist ;
         playlist_uri <- uri ;
-        Mutex.unlock mylock
+        (* The only case where keeping the old index is safe and makes sense *)
+        if not (random = Normal && index_played < Array.length !playlist) then
+          index_played <- -1 ;
+        Mutex.unlock mylock ;
+        self#log#f 3
+          "Successfully loaded a playlist of %d tracks."
+          (Array.length !playlist)
       end
 
   (* [reloading] avoids two reloadings being prepared at the same time. *)
   method reload_playlist ?new_reload ?new_random ?new_playlist_uri () =
-    if Mutex.try_lock reloading then
-      Tutils.Task.create
-        (fun () ->
-          self#reload_playlist_internal
-          new_reload new_random new_playlist_uri ;
-          Mutex.unlock reloading ;
-          Tutils.Task.Finish)
+      Duppy.Task.add Tutils.scheduler 
+        { Duppy.Task.
+	    priority = Tutils.Maybe_blocking ;
+            events   = [`Delay 0.] ;
+            handler  = (fun _ ->
+              if Mutex.try_lock reloading then
+                begin
+                  self#reload_playlist_internal
+                    new_reload new_random new_playlist_uri ;
+                  Mutex.unlock reloading 
+                end ;
+              []) }
 
   method reload_playlist_nobg ?new_reload ?new_random ?new_playlist_uri () =
     if Mutex.try_lock reloading then
@@ -217,7 +255,7 @@ object (self)
           if round_c <= 0 then
             self#reload_playlist ()
 
-  method get_next_request : Request.t option =
+  method get_next_request : Request.audio Request.t option =
     Mutex.lock mylock ;
     if !playlist = [||] then
       ( self#reload_update true ;
@@ -260,24 +298,24 @@ object (self)
       let x = Filename.basename playlist_uri in
       match x with
         | "." ->
-         begin
-           match List.rev (Pcre.split ~pat:"/" playlist_uri) with
-             | e :: _ -> e
-             | [] -> Filename.dirname playlist_uri
-         end
-       | _ -> x
+            begin
+              match List.rev (Pcre.split ~pat:"/" playlist_uri) with
+                | e :: _ -> e
+                | [] -> Filename.dirname playlist_uri
+            end
+        | _ -> x
     in
     let id =
       match base_name with
         | "." -> (* Definitly avoid this *)
-          Printf.sprintf "playlist-%s" playlist_uri
-       | _ -> base_name
+            Printf.sprintf "playlist-%s" playlist_uri
+        | _ -> base_name
     in
     self#set_id ~definitive:false id ;
     self#reload_playlist_nobg () ;
     if ns = [] then
       ns <- Server.register [self#id] "playlist" ;
-    Server.add ~ns "next"
+    Server.add ~ns "next" ~descr:"Return up to 10 next URIs to be played."
       (* The next command returns up to 10 next URIs to be played.
        * We cannot return request ids cause we create requests at the last
        * moment. We get requests from Request_source.* classes, from which
@@ -309,17 +347,17 @@ object (self)
           (fun i -> !playlist.((index_played+i+1) mod (Array.length !playlist)))
     | Randomize ->
         Array.init
-          (min n (Array.length !playlist)-index_played-1)
+          (min n (Array.length !playlist - index_played - 1))
           (fun i -> !playlist.(index_played+i+1))
     | Random -> [||])
 
 end
 
 (** Standard playlist, with a queue. *)
-class playlist reload_meth rand_meth uri length default_duration timeout =
+class playlist mime reload_meth rand_meth uri length default_duration timeout =
 object
 
-  inherit vplaylist reload_meth rand_meth uri timeout as pl
+  inherit vplaylist mime reload_meth rand_meth uri timeout as pl
   inherit Request_source.queued ~length ~default_duration ~timeout () as super
 
   method reload_playlist_internal a b c =
@@ -339,10 +377,10 @@ end
 
 (** Safe playlist, without queue and playing only local files,
   * which never fails. *)
-class safe_playlist reload_meth rand_meth local_playlist timeout = 
+class safe_playlist mime reload_meth rand_meth local_playlist timeout =
 object (self)
 
-  inherit vplaylist reload_meth rand_meth local_playlist timeout as pl
+  inherit vplaylist mime reload_meth rand_meth local_playlist timeout as pl
   inherit Request_source.unqueued as super
 
   method wake_up =
@@ -379,28 +417,36 @@ let () =
     [ "mode",
       Lang.string_t,
       (Some (Lang.string "randomize")),
-      (Some "normal|random|randomize") ;
-
+      Some "Play the files in the playlist either in the order (\"normal\" \
+            mode), or shuffle the playlist and play it in this order \
+            (\"randomize\" mode), or pick a random file in the playlist \
+            each time (\"random\" mode)." ;
       "reload",
       Lang.int_t,
-      (Some (Lang.int 0)),
-      Some ("Amount of time (in seconds or rounds) before which "^
-            "the playlist is reloaded; 0 means never.") ;
+      Some (Lang.int 0),
+      Some "Amount of time (in seconds or rounds) before which \
+            the playlist is reloaded; 0 means never." ;
 
       "reload_mode",
       Lang.string_t,
-      (Some (Lang.string "seconds")),
-      (Some ("rounds|seconds: unit of the 'reload' parameter.")) ;
+      Some (Lang.string "seconds"),
+      Some "Unit of the reload parameter, either 'rounds' or 'seconds'." ;
+
+      "mime_type",
+      Lang.string_t,
+      Some (Lang.string ""),
+      Some "Default MIME type for the playlist. \
+            Empty string means automatic detection." ;
 
       "timeout",
       Lang.float_t,
-      (Some (Lang.float 20.)),
-      (Some ("Timeout (in sec.) for a single download.")) ;
+      Some (Lang.float 20.),
+      Some "Timeout (in seconds) for a single download." ;
 
       "",
       Lang.string_t,
       None,
-      (Some "URI where to find the playlist.") ]
+      Some "URI where to find the playlist." ]
   in
   let reload_of i s =
     let arg = Lang.to_int i in
@@ -430,31 +476,33 @@ let () =
       ~descr:"Loop on a playlist of URIs."
       (Request_source.queued_proto@proto)
       (fun params ->
-         let reload,random,timeout,uri =
+         let reload,random,timeout,mime,uri =
            let e v = List.assoc v params in
              (reload_of (e "reload") (e "reload_mode")),
              (random_of (e "mode")),
              (Lang.to_float (e "timeout")),
+             (Lang.to_string (e "mime_type")),
              (Lang.to_string (e ""))
          in
          let l,d,t = Request_source.extract_queued_params params in
-           ((new playlist reload random uri l d t):>source)) ;
+           ((new playlist mime reload random uri l d t):>source)) ;
 
     Lang.add_operator "playlist.safe"
       ~category:Lang.Input
-      ~descr:("Loop on a playlist of local files, "^
-              "and never fail. In order to do so, it has to check "^
-              "every file at the loading, so the streamer startup may take "^
-              "a few seconds. To avoid this, use a standard playlist, "^
-              "and put only a few local files in a default safe_playlist "^
-              "in order to ensure the liveness of the streamer.")
+      ~descr:"Loop on a playlist of local files, \
+              and never fail. In order to do so, it has to check \
+              every file at the loading, so the streamer startup may take \
+              a few seconds. To avoid this, use a standard playlist, \
+              and put only a few local files in a default safe_playlist \
+              in order to ensure the liveness of the streamer."
       proto
       (fun params ->
-         let reload,random,timeout,uri =
+         let reload,random,timeout,mime,uri =
            let e v = List.assoc v params in
              (reload_of (e "reload") (e "reload_mode")),
              (random_of (e "mode")),
              (Lang.to_float (e "timeout")),
+             (Lang.to_string (e "mime_type")),
              (Lang.to_string (e ""))
          in
-           ((new safe_playlist reload random uri timeout):>source))
+           ((new safe_playlist mime reload random uri timeout):>source))
