@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -37,18 +37,23 @@ object (self)
 
   method get_next_request =
     try
-      Some (Rqueue.shift queue)
+      let r = Rqueue.shift queue in
+      Request.add_log r "Entering the primary queue." ;
+      Request.set_root_metadata r "queue" "primary" ;
+      Some r
     with
-      | Rqueue.Not_found -> None
+      | Rqueue.Not_found ->  None
 
   val mutable ns = []
 
-  method copy_queue =
+  method copy_queue_init q =
     Rqueue.fold
       (fun l r -> r::l)
-      super#copy_queue
+      q
       queue
 
+  method copy_queue = self#copy_queue_init super#copy_queue
+
   method wake_up activation =
     super#wake_up activation ;
     if ns = [] then
@@ -57,21 +62,36 @@ object (self)
       match self#create_request req with
       | Some req ->
           f req ;
+          Request.add_log req "Entering the secondary queue." ;
+          Request.set_root_metadata req "queue" "secondary" ;
           self#notify_new_request ;
           (string_of_int (Request.get_id req))
       | None -> "Unable to create a request!"
     in
       Server.add ~ns "push" ~usage:"push <uri>"
+                 ~descr:"Push a new request in the queue."
         (make_n_add (Rqueue.push queue)) ;
-      Server.add ~ns "queue"
-        (fun _ ->
+    let print_queue q = 
            String.concat " "
              (List.map
                 (fun r -> string_of_int (Request.get_id r))
-                (List.rev self#copy_queue))) ;
+                (List.rev q))
+    in
+      Server.add ~ns "queue"
+        (fun _ -> print_queue self#copy_queue)
+                 ~descr:"Display current queue content for \
+		         both primary and secondary queues.";
+      Server.add ~ns "primary_queue"
+        (fun _ -> print_queue super#copy_queue)
+                 ~descr:"Display current queue content for \
+                         the primary queue.";
+      Server.add ~ns "secondary_queue"
+        (fun _ -> print_queue (self#copy_queue_init []))
+                 ~descr:"Display current queue content for \
+                         the secondary queue.";
       (* Since the queue command gives not only the pending queue,
        * it can be useful to have the size of our queue *)
-      Server.add ~ns "pending_length"
+      Server.add ~ns "pending_length" ~descr:"Return the length of the secondary queue."
         (fun _ ->
            string_of_int (Rqueue.length queue)) ;
       Server.add ~ns "insert" ~usage:"insert <pos> <uri>"
@@ -81,7 +101,8 @@ object (self)
              let uri = Str.matched_group 2 a in
                make_n_add (Rqueue.insert queue pos) uri
            else
-             "Usage: insert <pos> <uri>") ;
+             "Usage: insert <pos> <uri>") 
+	        ~descr:"Insert <uri> at position <pos> in the secondary queue." ;
       Server.add ~ns "remove" ~usage:"remove <rid>"
         (fun a ->
            let id = int_of_string a in
@@ -92,8 +113,10 @@ object (self)
                  Request.destroy req ;
                  "OK"
              with
-             | Rqueue.Not_found -> "No such request in my queue") ;
-      Server.add ~ns "move" ~usage:"move <rid> <pos>"
+             | Rqueue.Not_found -> "No such request in my queue") 
+	        ~descr:"Remove request <rid> from the secondary queue." ;
+      Server.add ~ns "move" ~usage:"move <rid> <pos>" 
+                 ~descr:"Move request <rid> in the secondary queue."
         (fun a ->
            if Str.string_match move_re a 0 then
              let rid = int_of_string (Str.matched_group 1 a) in
