@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -61,6 +61,7 @@ object (self)
 	else
 	  ( Request.add_log out "Entering the primary queue." ;
             Request.set_root_metadata out "2nd_queue_pos" "0" ;
+	    Request.set_root_metadata out "queue" "primary" ;
 	    Some out )
     with
       | Queue.Empty -> Mutex.unlock reqlock ; None
@@ -76,16 +77,19 @@ object (self)
            (Queue.length requests)) ;
       Request.set_root_metadata req "2nd_queue_pos"
         (string_of_int (Queue.length requests)) ;
+      Request.set_root_metadata req "queue" "secondary" ;
       Mutex.unlock reqlock ;
       self#notify_new_request
 
-  (** Get a copy of the resolved (primary) and unresolved (secondary) queues. *)
-  method copy_queue =
+  method copy_queue_init q =
     Mutex.lock reqlock ;
-    let q = Queue.fold (fun l r -> r::l) queued#copy_queue requests in
+    let q = Queue.fold (fun l r -> r::l) q requests in
       Mutex.unlock reqlock ;
       q
 
+  (** Get a copy of the resolved (primary) and unresolved (secondary) queues. *)
+  method copy_queue = self#copy_queue_init queued#copy_queue
+
   val mutable ns = []
 
   method wake_up activation =
@@ -103,14 +107,29 @@ object (self)
                  (string_of_int (Oo.id self)) ;
                self#push_request req ;
                (string_of_int id)
-         | None -> "Unable to create a request!" ) ;
+         | None -> "Unable to create a request!" ) 
+               ~descr:"Push a new request in the queue." ;
+    let print_queue q =
+           String.concat " "
+             (List.map
+                (fun r -> string_of_int (Request.get_id r))
+                (List.rev q))
+    in
     Server.add ~ns "queue"
-      (fun _ ->
-         String.concat " "
-           (List.map
-              (fun r -> string_of_int (Request.get_id r))
-              (List.rev self#copy_queue))) ;
+      (fun _ -> print_queue self#copy_queue)
+               ~descr:"Display current queue content for both primary and \
+                       secondary queue." ;
+    Server.add ~ns "primary_queue"
+      (fun _ -> print_queue queued#copy_queue)
+               ~descr:"Display current queue content for the primary \
+                       queue." ;
+    Server.add ~ns "secondary_queue"
+      (fun _ -> print_queue (self#copy_queue_init []))
+               ~descr:"Display current queue content for the seconary \
+                       queue." ;
+
     Server.add ~ns "ignore" ~usage:"ignore <rid>"
+      ~descr:"Indicate that request <rid> should not be played (sets the skip metadata to true)."
       (fun s ->
          ( let id = int_of_string s in
              match Request.from_id id with
@@ -123,6 +142,7 @@ object (self)
                    ( Request.set_root_metadata r "skip" "true" ;
                      "OK" ))) ;
     Server.add ~ns "consider" ~usage:"consider <rid>"
+      ~descr:"Cancel the effect of ignore on a request."
       (fun s ->
          ( let id = int_of_string s in
              match Request.from_id id with
