@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -109,11 +109,17 @@ class dynamic (f:Lang.value) length default_duration timeout =
 object (self)
   inherit Request_source.queued ~length ~default_duration ~timeout ()
   method get_next_request =
-    match Lang.to_request (Lang.eval (Lang.apply f [])) with
-      | None -> None
-      | Some req ->
-          Request.set_root_metadata req "source" self#id ;
-          Some req
+    try
+      match Lang.to_request (Lang.apply f []) with
+        | None -> None
+        | Some req ->
+              Request.set_root_metadata req "source" self#id ;
+              Some req
+    with
+      | e ->
+          log#f 2 "Couldn't get an audio request! \
+            Make sure you did not override audio=false in request.create()." ;
+          raise e
 end
 
 let () =
