@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ object (self)
 
   (** [get_next_file] is supposed to return "quickly".
     * This means that no resolving should be done here. *)
-  method virtual get_next_file : Request.t option
+  method virtual get_next_file : Request.audio Request.t option
 
   val mutable remaining = 0
 
@@ -50,7 +50,10 @@ object (self)
         | Some (request,_,close) ->
             begin match Request.get_filename request with
               | None ->
-                  self#log#f 1 "Finished with a non-existent file ?! Something may have been moved or destroyed during decoding. It is VERY dangerous, avoid it !"
+                  self#log#f 1
+                    "Finished with a non-existent file ?! \
+                     Something may have been moved or destroyed \
+                     during decoding. It is VERY dangerous, avoid it !"
               | Some f -> self#log#f 3 "Finished with %S" f
             end ;
             close () ;
@@ -113,9 +116,9 @@ object (self)
           | Some (req,get_frame,_) ->
               if send_metadata then begin
                 Request.on_air req ;
+                let m = Request.get_all_metadata req in
                 Frame.set_metadata buf
-                  (Frame.position buf)
-                  (Request.get_all_metadata req) ;
+                  (Frame.position buf) m;
                 send_metadata <- false
               end ;
               get_frame buf
@@ -139,6 +142,12 @@ object (self)
 
 end
 
+(* Private types for request resolutions *)
+type resolution = Empty | Retry | Finished
+
+(* Scheduler priority for request resolutions. *)
+let priority = Tutils.Maybe_blocking
+
 (** Same thing, with a queue in which we prefetch files,
   * which requests are given by [get_next_request].
   * Heuristical settings determining how the source feeds its queue:
@@ -149,7 +158,7 @@ class virtual queued ?(length=60.) ?(default_duration=30.) ?(timeout=20.) () =
 object (self)
   inherit unqueued as super
 
-  method virtual get_next_request : Request.t option
+  method virtual get_next_request : Request.audio Request.t option
 
   (** Management of the queue of files waiting to be played. *)
   val min_queue_length = Fmt.ticks_of_seconds length
@@ -158,49 +167,122 @@ object (self)
   val mutable queue_length = 0 (* Frames *)
   val mutable resolving = None
 
-  (** Launch the feeding task when the source wakes up,
-    * finish it when the source goes to sleep. *)
-  val mutable sleeping = true
-  val mutable task_id = None
+  (** Asynchronous task for waking up the feeding process.. *)
+  val run_m = Mutex.create ()
+  val mutable do_run = false
+  val mutable wake_task = None 
+
+  (** State should be `Sleeping on awakening, and is then turned to `Running.
+    * Eventually #sleep puts it to `Tired, then waits for it to be `Sleeping,
+    * meaning that the feeding tasks exited. *)
+  val mutable state = `Sleeping
+  val state_lock = Mutex.create ()
+  val state_cond = Condition.create ()
 
   method private wake_up activation =
-    sleeping <- false ;
-    Tutils.Task.create ~k:(fun i -> task_id <- Some i) self#feed_queue
+    Tutils.mutexify state_lock
+      (fun () ->
+         assert (state = `Sleeping) ;
+         state <- `Running) () ;
+    Mutex.lock run_m; do_run <- true; Mutex.unlock run_m ;
+    wake_task <- 
+      Some 
+        (Duppy.Async.add Tutils.scheduler ~priority
+          (fun () ->  Mutex.lock run_m ;
+                      if do_run then begin
+                        Duppy.Task.add Tutils.scheduler
+                         { Duppy.Task.
+                           priority = priority ;
+                           events   = [`Delay 0.] ;
+                           handler  = (fun _ -> self#feed_queue () ; []) } ;
+                        do_run <- false
+                      end ;
+		      Mutex.unlock run_m))
 
   method private sleep =
-    sleeping <- true ;
-    task_id <- None ;
+    Tutils.mutexify state_lock
+      (fun () ->
+         assert (state = `Running) ;
+         state <- `Tired) () ;
+    Tutils.wait state_cond state_lock (fun () -> state = `Sleeping) ;
     super#sleep ;
-    try
+    begin try
+      Mutex.lock qlock ;
       while true do
         let (_,req) = Queue.take retrieved in
           Request.destroy req
       done
-    with Queue.Empty -> ()
+    with e -> Mutex.unlock qlock ; if e <> Queue.Empty then raise e end ;
+    let task = Utils.get_some wake_task in
+      wake_task <- None ;
+      Duppy.Async.stop task
 
   (** This method should be called whenever the feeding task has a new
     * opportunity to feed the queue, in case it is sleeping. *)
   method private notify_new_request =
-    match task_id with
-      | None -> assert false
-      | Some id -> Tutils.Task.wake_up id
+    match wake_task with
+      | Some task -> Duppy.Async.wake_up task
+      | None -> ()
+
+  (** A function that returns delays for tasks, making sure that these tasks
+    * don't repeat too fast.
+    * The current scheme is to return 0. as long as there are no more than
+    * [max] consecutive occurences separated by less than [delay], otherwise
+    * return [delay]. *)
+  val adaptative_delay =
+    let last   = ref 0. in
+    let excess = ref 0  in
+    let delay = 2. in
+    let max   = 3  in
+    let next () =
+      let now = Unix.gettimeofday () in
+        if now -. !last < delay then incr excess else excess := 0 ;
+        last := now ;
+        if !excess >= max then delay else 0.
+    in
+      next
 
   (** The body of the feeding task *)
-  method private feed_queue () =
-    if sleeping then Tutils.Task.Finish else
-      (* We keep running as long as we want more audio in the queue
-       * and #prefetch believes it can feed the queue *)
-      if queue_length < min_queue_length && self#prefetch then
-        Tutils.Task.Yield self#feed_queue
-      else
-        Tutils.Task.Sleep self#feed_queue
+  method private feed_queue () : unit =
+    let put_on_sleep () =
+      Mutex.lock run_m ;
+      do_run <- true ;
+      Mutex.unlock run_m
+    in
+    (* If the test fails, the task ends.. *)
+    if
+      Tutils.mutexify state_lock
+        (fun () ->
+           if state <> `Tired then true else begin
+             state <- `Sleeping ;
+             Condition.signal state_cond ;
+             false
+           end) ()
+    then
+      if queue_length < min_queue_length then
+        match self#prefetch with
+          | Finished ->
+              Duppy.Task.add Tutils.scheduler
+                { Duppy.Task.
+                    priority = priority ;
+                    events   = [`Delay 0.] ;
+                    handler  =  (fun _ -> self#feed_queue (); []) }
+          | Retry ->
+              (* Reschedule the task later *)
+              Duppy.Task.add Tutils.scheduler
+                { Duppy.Task.
+                    priority  = priority ;
+                    events   = [`Delay (adaptative_delay ())] ;
+                    handler  = (fun _ -> self#feed_queue (); []) }
+          | Empty -> put_on_sleep ()
+      else put_on_sleep ()
 
   (** Try to feed the queue with a new request.
     * Return false if there was no new request to try,
     * true otherwise, whether the request was fetched successfully or not. *)
   method private prefetch =
     match self#get_next_request with
-      | None -> false
+      | None -> Empty
       | Some req ->
           resolving <- Some req ;
           begin match Request.resolve req timeout with
@@ -211,20 +293,23 @@ object (self)
                         (try float_of_string f with _ -> default_duration)
                     | None -> default_duration
                 in
-                let len = int_of_float (len *. float (Fmt.ticks_per_second())) in
+                let len =
+                  int_of_float (len *. float (Fmt.ticks_per_second()))
+                in
                   Mutex.lock qlock ;
                   Queue.add (len,req) retrieved ;
                   self#log#f 4 "queue length %d+=%d (rid %d)"
                     queue_length len (Request.get_id req) ;
                   queue_length <- queue_length + len ;
                   Mutex.unlock qlock ;
-                  resolving <- None
+                  resolving <- None ;
+                  Finished
               | Request.Failed (* Failure of resolving or decoding *)
               | Request.Timeout ->
                   resolving <- None ;
                   Request.destroy req ;
-          end ;
-          true
+                  Retry
+          end
 
   (** Provide the unqueued [super] with resolved requests. *)
   method private get_next_file =
@@ -234,7 +319,6 @@ object (self)
         let len,f = Queue.take retrieved in
           self#log#f 4 "queue length %d-=%d" queue_length len ;
           queue_length <- queue_length - len ;
-          self#notify_new_request ;
           Some f
       with
         | Queue.Empty ->
@@ -242,6 +326,7 @@ object (self)
             None
     in
       Mutex.unlock qlock ;
+      self#notify_new_request ;
       ans
 
   method copy_queue =
