@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,23 +26,23 @@ class virtual unqueued :
 object
   (** [get_next_file] is the only thing you've got to define,
     * it's supposed to return "quickly" as it is run in the Root thread. *)
-  method virtual get_next_file : Request.t option
+  method virtual get_next_file : Request.audio Request.t option
 
   inherit Source.source
   method is_ready : bool
   method private get_frame : Frame.t -> unit
   method abort_track : unit
-  method copy_queue : Request.t list
+  method copy_queue : Request.audio Request.t list
   method remaining : int
 end
 
 class virtual queued :
   ?length:float -> ?default_duration:float -> ?timeout:float -> unit ->
 object
-  method copy_queue : Request.t list
+  method copy_queue : Request.audio Request.t list
 
   (** You should only define this. *)
-  method virtual get_next_request : Request.t option
+  method virtual get_next_request : Request.audio Request.t option
 
   (** This method should be called whenever the feeding task gets
     * a new opportunity to add more data into the queue. *)
@@ -51,8 +51,8 @@ object
   inherit unqueued
 
   (** Everything you need is defined. Dont touch. *)
-  method private get_next_file : Request.t option
+  method private get_next_file : Request.audio Request.t option
 end
 
-val queued_proto : Lang.operator_proto
-val extract_queued_params : Lang.parameters -> float*float*float
+val queued_proto : Lang.proto
+val extract_queued_params : Lang.env -> float*float*float
