@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 (** A [plug] is something where plug-ins plug.
   We build [plug] on the top of [Doc.item]. *)
 
-class ['a] plug (name:string) (doc:string) (insensitive:bool) =
+class ['a] plug name doc insensitive duplicates =
 object (self)
   inherit Doc.item doc
 
@@ -36,8 +36,14 @@ object (self)
       | _, None -> Doc.trivial "(no doc)"
       | _, (Some s) -> Doc.trivial s
     in
-      subsections <- (plugin,doc)::subsections ;
-      plugins <- (plugin,v)::plugins
+      if duplicates then begin
+        subsections <- (plugin,doc)::subsections ;
+        plugins <- (plugin,v)::plugins
+      end else begin
+        subsections <-
+          (plugin,doc)::(List.filter (fun (k,_) -> k<>plugin) subsections) ;
+        plugins <- (plugin,v)::(List.filter (fun (k,_) -> k<>plugin) plugins)
+      end
 
   method is_registered a = List.mem_assoc a plugins
   method keys = List.fold_left (fun l (k,v) -> k::l) [] plugins
@@ -56,10 +62,10 @@ end
 
 let plugs = new Doc.item "All the plugs"
 
-let create ?insensitive ?doc plugname =
+let create ?(duplicates=true) ?insensitive ?doc plugname =
   let insensitive = match insensitive with Some true -> true | _ -> false in
   let doc = match doc with None -> "(no doc)" | Some d -> d in
-  let plug = new plug plugname doc insensitive in
+  let plug = new plug plugname doc insensitive duplicates in
     plugs#add_subsection plugname (plug:>Doc.item) ;
     plug
 
