@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -107,7 +107,6 @@ let send ?(nosync=false) t b =
   if _send t.session (AFrame.to_s16le b) <= 0 then raise Error
 
 let recv ?(nosync=false) t b =
-  AFrame.free_metadata b ;
   if not nosync then sync t ;
   let s = meta_recv t.session in
     if s <> "" then begin
@@ -118,9 +117,13 @@ let recv ?(nosync=false) t b =
             (int_of_string (Hashtbl.find h "timestamp")) h
         with Not_found -> assert false
     end ;
-    let buf = AFrame.to_s16le b in (* TODO: allocate a fresh buffer of the right size *)
+    (* TODO: allocate a fresh buffer of the right size *)
+    let buf = AFrame.to_s16le b in
     let ret = _recv t.session buf in
-      Float_pcm.from_s16le (AFrame.get_float_pcm b) 0 buf 0 (String.length buf / 4);
+      Float_pcm.from_s16le
+        (AFrame.get_float_pcm b) 0
+        buf 0
+        (String.length buf / 4);
       (* 1 means have_more, -1 means that an error occured *)
       match ret with
         | 1  -> raise Have_more
