@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -78,66 +78,13 @@ let conf_telnet_port =
 
 let log = Log.make ["server"]
 
-(* {1 Light general-purpose multitask server} *)
-
-type task =
-  | Read    of (Unix.file_descr * (unit -> unit))
-  | Write   of (Unix.file_descr * (unit -> unit))
-
-(* WARNING: TODO: There should always be at least one event waited for,
- * otherwise the scheduler will block forever. *)
-let scheduler init =
-  let read    = ref [] in
-  let write   = ref [] in
-  let special = ref [] in
-  let add set a = assert (not (List.mem a !set)) ; set := a :: !set in
-  let add = function
-    | Read  (s,f) -> add read  (s,f) ; add special s
-    | Write (s,f) -> add write (s,f) ; add special s
-  in
-  let filter  x = List.filter (fun s -> not (List.mem s x)) in
-  let filter2 x = List.filter (fun (s,_) -> not (List.mem s x)) in
-  let close s =
-    log#f 3 "Client left." ;
-    try
-      Unix.shutdown s Unix.SHUTDOWN_ALL ;
-      Unix.close s
-    with
-      | _ -> ()
-  in
-  let process set sockets =
-    (* Events are triggered on a list of sockets.
-     * Execute the corresponding handlers, and remove them from the set. *)
-    let f = List.map (fun s -> s, List.assoc s !set) sockets in
-      set := filter2 sockets !set ;
-      special := filter sockets !special ;
-      List.iter
-        (fun (s,f) -> try f () with _ -> close s)
-        f
-  in
-  let close_special r w x =
-      List.iter close x ;
-      read  := (filter2 x) !read ;
-      write := (filter2 x) !write ;
-      special := (filter x) !special ;
-      (filter x) r, (filter x) w
-  in
-    init add ;
-    while not !Root.shutdown do
-      let r,w,x =
-        Utils.select (List.map fst !read) (List.map fst !write) !special (-1.)
-      in
-      let r,w = close_special r w x in
-        process read r ;
-        process write w
-    done
-
 (* {1 Manage available commands and namespaces} *)
 
 type namespace = string list
 
 let namespaces = Hashtbl.create 10
-let commands : (string,(string->string)*string) Hashtbl.t = Hashtbl.create 50
+let commands : (string,(string->string)*string*string) Hashtbl.t =
+  Hashtbl.create 50
 
 (* First, get a fresh namespace *)
 let register ns kind =
@@ -157,10 +104,10 @@ let to_string = String.concat "."
 let rec prefix_ns cmd ns = to_string (ns@[cmd])
 
 (* Then add your commands to that namespace *)
-let add ~ns ?usage cmd handler =
+let add ~ns ?usage ~descr cmd handler =
   let usage = match usage with None -> cmd | Some u -> u in
   let usage = prefix_ns usage ns in
-    Hashtbl.add commands (prefix_ns cmd ns) (handler,usage)
+    Hashtbl.add commands (prefix_ns cmd ns) (handler,usage,descr)
 
 (* ... maybe remove them *)
 let remove ~ns cmd =
@@ -171,7 +118,7 @@ let usage () =
   let l = Hashtbl.fold (fun k v l -> (k,v)::l) commands [] in
   let l = List.sort compare l in
     List.fold_left
-      (fun s (k,(h,u)) -> s^(Printf.sprintf "\n| %s" u))
+      (fun s (k,(h,u,_)) -> s^(Printf.sprintf "\n| %s" u))
       "" l
 
 (** {1 Handling of a client} *)
@@ -181,10 +128,28 @@ exception Exit
 (** The very-builtin commands *)
 let () =
   let add = add ~ns:[] in
-    add "exit" (fun args -> raise Exit) ;
-    add "quit" (fun args -> raise Exit) ;
-    add "help" (fun _ -> "Available commands:" ^ (usage ())) ;
-    add "list" (fun _ ->
+    add "exit" ~descr:"Close current server session." (fun args -> raise Exit) ;
+    add "quit" ~descr:"Close current server session." (fun args -> raise Exit) ;
+    add "help" ~usage:"help [<command>]" 
+               ~descr:"Get information on available commands." 
+      (fun args -> 
+         try
+            let args = 
+              Pcre.substitute ~pat:"\\s*" 
+                              ~subst:(fun _ -> "") args
+            in
+            let (_,us,d) = Hashtbl.find commands args in
+	    Printf.sprintf
+            "\nHelp for command %s.\n\nUsage: %s\n  %s"
+	    args us d
+         with
+          | Not_found ->
+              (if args <> "" then "No such command: " ^ args ^ "\n" else "")
+               ^ "Available commands:" ^ (usage ()) ^ "\n\n"
+               ^ "Type \"help <command>\" for more information.") ;
+    add "list" 
+        ~descr:"Get the list of available operators with their interfaces." 
+               (fun _ ->
                   String.concat "\n"
                     (Hashtbl.fold
                        (fun k v s ->
@@ -202,6 +167,7 @@ let exec s =
   in
     if Hashtbl.mem commands s then
       try
+        let fst (x,_,_) = x in
         (fst (Hashtbl.find commands s)) args
       with
         | Exit -> raise Exit
@@ -209,55 +175,60 @@ let exec s =
     else
       raise Not_found
 
-let handle_client add c =
-  (* Write answer [s] and move on to step [k] *)
-  let write s k =
-    ignore (Unix.write c s 0 (String.length s)) ;
-    add k
+let priority = Tutils.Non_blocking
+
+let handle_client socket =
+  let on_error _ =
+    log#f 3 "Client left." ;
+    try
+      Unix.shutdown socket Unix.SHUTDOWN_ALL ;
+      Unix.close socket
+    with
+      | _ -> ()
   in
-  (* Process the command [s] and move on to step [k] *)
-  let process s k =
-    let s =
-      if s.[(String.length s) - 1] = '\r' then
-        String.sub s 0 ((String.length s)-1)
-      else
-        s
+  (* Read and process lines *)
+  let marker = Duppy.Io.Split "[\r\n]+" in
+  let recursive = false in
+  (* Process the command [s] and start a new reading poll *)
+  let rec process l =
+    let l,init =
+      match List.rev l with
+        | []
+        | _ :: [] -> assert false (* Should not happen *)
+        | e :: l -> List.rev l,e
     in
-    let answer =
-      try exec s with
-        | Not_found ->
-            "ERROR: unknown command, type \"help\" to get a list of commands."
-    in
-    let answer =
+    let exit = ref false in
+    let process_elem s =
+      let answer =
+        try exec s with
+          | Not_found ->
+              "ERROR: unknown command, type \"help\" to get a list of commands."
+          | Exit -> exit := true ; "Bye!"
+      in
       let len = String.length answer in
         if len > 0 && answer.[len - 1] = '\n' then
           answer^"END\n"
         else
           answer^"\nEND\n"
     in
-      add (Write (c,(fun () -> write answer k)))
+    let l = List.map process_elem l in
+    let answer = String.concat "\r\n" l in
+    let exec () =
+      if !exit then
+        on_error ()
+      else
+        Duppy.Io.read ~priority ~recursive ~on_error ~init
+          Tutils.scheduler socket marker process
+    in
+    Duppy.Io.write ~priority ~exec ~on_error
+      Tutils.scheduler socket answer
   in
-  (* Read and process lines *)
-  let rec f acc () =
-    let buf = String.make 100 ' ' in
-    let input = Unix.read c buf 0 100 in
-    if input<=0 then failwith "eof" ;
-    let acc = acc ^ (String.sub buf 0 input) in
-      try
-        let index = String.index acc '\n' in
-        let line = String.sub acc 0 index in
-        let acc  =
-          String.sub acc (index+1) (String.length acc - index - 1)
-        in
-          process line (Read (c,f acc))
-      with
-        | Not_found -> add (Read (c,f acc))
-  in
-    add (Read (c,f ""))
+  Duppy.Io.read ~priority ~recursive ~on_error
+    Tutils.scheduler socket marker process
 
 (* {1 The server} *)
 
-let start_socket add =
+let start_socket () =
   let socket_path = Configure.subst_vars (conf_socket_path#get) in
   let socket_name = Filename.basename socket_path in
   let socket_dir = Filename.dirname socket_path in
@@ -265,14 +236,21 @@ let start_socket add =
   let rights = conf_socket_perms#get in
   let max_conn = 10 in
   let sock = socket PF_UNIX SOCK_STREAM 0 in
-  let rec incoming () =
-    add (Read (sock,incoming)) ;
-    try
-      let (s,caller) = accept sock in
-      handle_client add s ;
-      log#f 3 "New client on %s" socket_name
-    with e ->
-      log#f 2 "Failed to accept new client: %S" (Printexc.to_string e)
+  (* Set TCP_NODELAY on the socket *)
+  Liq_sockets.set_tcp_nodelay sock true ;
+  let rec incoming _ =
+    begin
+      try
+        let (socket,caller) = accept sock in
+          log#f 3 "New client on %s" socket_name ;
+          handle_client socket
+      with e ->
+        log#f 2 "Failed to accept new client: %S" (Printexc.to_string e)
+    end ;
+    [{ Duppy.Task.
+         priority = priority ;
+         events = [`Read sock] ;
+         handler = incoming }]
   in
     setsockopt sock SO_REUSEADDR true ;
     (* Try to close the socket if exists.. *)
@@ -294,9 +272,13 @@ let start_socket add =
                  Unix.unlink socket_path)) ;
     chmod socket_path rights ;
     listen sock max_conn ;
-    add (Read (sock,incoming))
+    Duppy.Task.add Tutils.scheduler
+      { Duppy.Task.
+          priority = priority ;
+          events = [`Read sock] ;
+          handler = incoming }
 
-let start_telnet add =
+let start_telnet () =
   let port = conf_telnet_port#get in
   let bind_addr_inet =
     inet_addr_of_string (conf_telnet_bind_addr#get)
@@ -304,24 +286,31 @@ let start_telnet add =
   let bind_addr = ADDR_INET(bind_addr_inet, port) in
   let max_conn = 10 in
   let sock = socket PF_INET SOCK_STREAM 0 in
-  let rec incoming () =
-    add (Read (sock,incoming)) ;
-    try
-      let (s,caller) = accept sock in
-      let ip =
-      let a = match caller with
-          | ADDR_INET (a,_) -> a
-          | _ -> assert false
-      in
+  (* Set TCP_NODELAY on the socket *)
+  Liq_sockets.set_tcp_nodelay sock true ;
+  let rec incoming _ =
+    begin
       try
-        (gethostbyaddr a).h_name
-      with
-        | Not_found -> string_of_inet_addr a
-      in
-      handle_client add s ;
-      log#f 3 "New client: %s" ip
-    with e ->
-    log#f 2 "Failed to accept new client: %S" (Printexc.to_string e)
+        let (socket,caller) = accept sock in
+        let ip =
+        let a = match caller with
+            | ADDR_INET (a,_) -> a
+            | _ -> assert false
+        in
+        try
+          (gethostbyaddr a).h_name
+        with
+          | Not_found -> string_of_inet_addr a
+        in
+        log#f 3 "New client: %s" ip ;
+        handle_client socket;
+      with e ->
+      log#f 2 "Failed to accept new client: %S" (Printexc.to_string e)
+    end ;
+    [{ Duppy.Task.
+         priority = priority ;
+         events   = [`Read sock] ;
+         handler  = incoming }]
     in
     setsockopt sock SO_REUSEADDR true ;
     begin try bind sock bind_addr with
@@ -329,19 +318,21 @@ let start_telnet add =
           failwith (Printf.sprintf "port %d already taken" port)
     end ;
     listen sock max_conn ;
-    add (Read (sock,incoming))
+    Duppy.Task.add Tutils.scheduler
+      { Duppy.Task.
+          priority = priority ;
+          events   = [`Read sock] ;
+          handler  = incoming }
 
 let start () =
   Sys.set_signal Sys.sigpipe Sys.Signal_ignore ;
   let telnet = conf_telnet#get in
   let socket = conf_socket#get in
-  if telnet || socket then
-    ignore
-      (Tutils.create scheduler
-         (fun add ->
-            if telnet then start_telnet add;
-            if socket then start_socket add)
-         "command server")
+  if telnet || socket then begin
+    Tutils.need_non_blocking_queue () ;
+    if telnet then start_telnet () ;
+    if socket then start_socket ()
+  end
 
 let () = ignore (Dtools.Init.at_start start)
 
