@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,22 +20,31 @@
 
  *****************************************************************************)
 
+(** Where the server commands are registered, and the server interfaces are
+  * created and controlled. *)
+
 open Dtools
 
 (** {2 Configuration parameters} *)
 
 (** The configuration of the server. *)
 val conf : Conf.ut
+
 (** Should a socket server be created? *)
 val conf_socket : bool Conf.t
+
 (** Path of the socket. *)
 val conf_socket_path : string Conf.t
+
 (** Permissions of the socket. **)
 val conf_socket_perms : int Conf.t
+
 (** Should a telnet server be created? *)
 val conf_telnet : bool Conf.t
+
 (** Address of the telnet server. *)
 val conf_telnet_bind_addr : string Conf.t
+
 (** Port of the telnet server. *)
 val conf_telnet_port : int Conf.t
 
@@ -45,6 +54,7 @@ val conf_telnet_port : int Conf.t
   * commands to the server should first get a new unique namespace, in which the
   * commands will be added. *)
 type namespace = string list
+
 (** Get a string representation of a namespace. *)
 val to_string : namespace -> string
 
@@ -55,23 +65,17 @@ val to_string : namespace -> string
   * this is. In liquidsoap, it tells which kind of operator owns the namespace,
   * and hence which commands should be expected in it. *)
 val register : namespace -> string -> namespace
-(** [add command f] adds a new command [command] in a given namespace. When
-  * the command is called, the function [f] is executed with the arguments of
-  * the command as parameter. The return value of [f] is then displayed. *)
-val add : ns:namespace -> ?usage:string -> string -> (string -> string) -> unit
+
+(** [add ~ns ~descr command f] adds a new command [command] in a given namespace ~ns. 
+  * When the command is called, the function [f] is executed with the argument of
+  * the command as parameter. The return value of [f] is then displayed. ~descr is 
+  * the command description. *)
+val add : ns:namespace -> ?usage:string -> descr:string ->
+          string -> (string -> string) -> unit
+
 (** Remove a command from the server. *)
 val remove : ns:namespace -> string -> unit
 
 (** [exec command] executes the [command], returns its result.
   * @raise Not_found if the command does not exist.  *)
 val exec : string -> string
-
-(** {2 Multi-socket event engine} *)
-
-(** A task for the scheduler. *)
-type task =
-  | Read    of (Unix.file_descr * (unit -> unit))
-  | Write   of (Unix.file_descr * (unit -> unit))
-
-(** [scheduler f] passes to [f] a way to add events. *)
-val scheduler : ((task -> unit) -> unit) -> unit
