@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,10 +22,7 @@
 
 (** Multithreading utilities *)
 
-(** Make a function work in critical section, protected by a given lock. *)
-val mutexify : Mutex.t -> ('a -> 'b) -> ('a -> 'b)
-
-(** Thread wrapper.
+(** {1 Thread wrapper}
   Give names to threads, and forbid them to raise an exception.
   The main process is expected to run raise_everything after having launched
   the needed threads. So the main process will sleep, until a thread
@@ -41,13 +38,27 @@ val join_all : unit -> unit
 (** Check if a thread is running. *)
 val running : string -> Thread.t -> bool
 
-module Task :
-sig
-  type task     = unit -> return_t
-  and  return_t = Sleep of task | Yield of task | Finish
+(** {1 Multi-tasking scheduler} *)
 
-  type id
+(** Priorities for the different scheduler usages. *)
+type priority =
+  | Blocking       (** For example a last.fm submission. *)
+  | Maybe_blocking (** Request resolutions vary a lot. *)
+  | Non_blocking   (** Non-blocking tasks like the server. *)
 
-  val create  : ?k:(id -> unit) -> task -> unit
-  val wake_up : id   -> unit
-end
+(** task scheduler *)
+val scheduler : priority Duppy.scheduler
+
+(** Ask for a special queue that treats exclusively non blocking tasks.
+  * This is need for things such as the server or harbor to run smoothly. *)
+val need_non_blocking_queue : unit -> unit
+
+(** {1 Misc} *)
+
+(** Waits for [f()] to become true on condition [c].
+  * The mutex [m] protecting data accessed by [f] is in the same state before
+  * and after the call. *)
+val wait : Condition.t -> Mutex.t -> (unit -> bool) -> unit
+
+(** Make a function work in critical section, protected by a given lock. *)
+val mutexify : Mutex.t -> ('a -> 'b) -> ('a -> 'b)
