@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2007 Savonet team
+  Copyright 2003-2008 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -56,6 +56,15 @@ let rec may_map f = function
       )
   | [] -> []
 
+let really_read fd buf ofs len =
+  let l = ref 0 in
+  let r = ref (-1) in
+    while !l < len && !r <> 0 do
+      r := Unix.read fd buf !l (len - !l);
+      l := !l + !r
+    done;
+    !l
+
 (** Perfect Fisher-Yates shuffle
   * (http://www.nist.gov/dads/HTML/fisherYatesShuffle.html). *)
 let randomize a =
@@ -138,6 +147,7 @@ let strftime str : string =
       "d", Printf.sprintf "%02d" t.Unix.tm_mday      ;
       "m", Printf.sprintf "%02d" (t.Unix.tm_mon + 1) ;
       "Y", string_of_int (t.Unix.tm_year + 1900)     ;
+      "w", string_of_int (t.Unix.tm_wday)            ;
       "%", "%" ]
   in
   let subst sub =
